//---------------------------------------------------------------------------
//
//  Module: winapp.c
//
//  Purpose: Demonstrates suspend/release with DOSXFER.386
//
//  Development Team:
//     Bryan A. Woodruff
//
//  History:   Date       Author      Comment
//              6/ 5/93   BryanW      Wrote it.
//
//---------------------------------------------------------------------------
//
//               Copyright (c) 1993 Woodruff Software Systems.
//                          All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "winapp.h"

HINSTANCE hInst ;

//------------------------------------------------------------------------
//  int WinMain
//
//  Description:
//     This is the main window procedure.
//
//  Parameters:
//     HINSTANCE hInstance
//        handle to this instance
//
//     HINSTANCE hPrevInstance
//        previous instances?
//
//     LPSTR lpCmdLine
//        command line parameters
//
//     int nCmdShow
//        how to show window
//
//  History:   Date       Author      Comment
//              6/ 5/93   BryanW      Wrote it.
//
//------------------------------------------------------------------------

int FAR PASCAL WinMain
(
   HINSTANCE   hInstance,
   HINSTANCE   hPrevInstance,
   LPSTR       lpCmdLine,
   int         nCmdShow
)
{
   MSG  msg ;

   if (!hPrevInstance)
      if (!InitApplication( hInstance ))
         return ( FALSE ) ;

   if (!InitInstance( hInstance, nCmdShow ))
      return ( FALSE ) ;

   while (GetMessage( &msg, NULL, NULL, NULL ))
   {
      TranslateMessage( &msg ) ;
      DispatchMessage( &msg ) ;
   }
   return ( msg.wParam ) ;

} // end of WinMain()

//------------------------------------------------------------------------
//  BOOL InitApplication
//
//  Description:
//     Application specific initialization.
//
//  Parameters:
//     HINSTANCE hInstance
//        handle to first instance.
//
//  History:   Date       Author      Comment
//              6/ 5/93   BryanW      Wrote it.
//
//------------------------------------------------------------------------

BOOL NEAR InitApplication
(
   HINSTANCE    hInstance
)
{
   WNDCLASS  wc ;

   wc.style =         NULL ;
   wc.lpfnWndProc =   MainWndProc ;
   wc.cbClsExtra =    0 ;
   wc.cbWndExtra =    0 ;
   wc.hInstance =     hInstance ;
   wc.hIcon =         LoadIcon( NULL, IDI_APPLICATION ) ;
   wc.hCursor =       LoadCursor( NULL, IDC_ARROW ) ;
   wc.hbrBackground = GetStockObject( WHITE_BRUSH ) ;
   wc.lpszMenuName =  "GenericMenu" ;
   wc.lpszClassName = "GenericWClass" ;

   return ( RegisterClass( &wc ) ) ;

} // end of InitApplication()

//------------------------------------------------------------------------
//  BOOL InitInstance
//
//  Description:
//     Performs instance initialization.
//
//  Parameters:
//     HINSTANCE hInstance
//        handle to this instance
//
//     int nCmdShow
//        how to show the window
//
//  History:   Date       Author      Comment
//              6/ 5/93   BryanW      Wrote it.
//
//------------------------------------------------------------------------

BOOL NEAR InitInstance
(
   HINSTANCE   hInstance,
   int         nCmdShow
)
{
   HWND  hWnd ;

   hInst = hInstance ;

   hWnd = CreateWindow( "GenericWClass", "Generic Sample Application",
                        WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT,
                        CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL,
                        hInstance, NULL ) ;

   if (!hWnd)
      return ( FALSE ) ;

   ShowWindow( hWnd, nCmdShow ) ;
   UpdateWindow( hWnd ) ;
   return ( TRUE ) ;

} // end of InitInstance()

//------------------------------------------------------------------------
//  LRESULT MainWndProc
//
//  Description:
//
//
//  Parameters:
//
//  History:   Date       Author      Comment
//
//------------------------------------------------------------------------

LRESULT FAR PASCAL MainWndProc
(
   HWND     hWnd,
   UINT     uMsg,
   WPARAM   wParam,
   LPARAM   lParam
)
{
   switch (uMsg)
   {
      case WM_CREATE:
         EnableNotification( hWnd ) ;
         break;

      case WM_NOTIFICATION:
      {
         char          szMsg[ 132 ] ;
         HDC           hDC ;
         NOTIFYSTRUCT  ns ;
         RECT          rc ;

         GetDataFromDOSXFER( &ns, lParam ) ;
         wsprintf( szMsg, "dwTime = %lu", ns.dwTime ) ;

         hDC = GetDC( hWnd ) ;
         SetTextColor( hDC, RGB( 0, 0, 0 ) ) ;
         SetBkColor( hDC, RGB( 255, 255, 255 ) ) ;
         rc.left = 10 ;
         rc.top = 10 ;
         rc.right = 640 ;
         rc.bottom = 100 ;
         FillRect( hDC, &rc, GetStockObject( WHITE_BRUSH ) ) ;
         DrawText( hDC, szMsg, lstrlen( szMsg ), &rc, DT_LEFT ) ;
         ReleaseDC( hWnd, hDC ) ;
      }
      break ;

      case WM_COMMAND:
         switch (wParam)
         {
            case IDM_ABOUT:
            {
               FARPROC  lpProcAbout ;

               lpProcAbout = MakeProcInstance( AboutDlgProc, hInst ) ;
               DialogBox( hInst, "AboutBox", hWnd, lpProcAbout ) ;
               FreeProcInstance( lpProcAbout ) ;
            }
            break ;
         }
         break ;

      case WM_DESTROY:
         DisableNotification() ;
         PostQuitMessage( 0 ) ;
         break ;

      default:
         return ( DefWindowProc( hWnd, uMsg, wParam, lParam ) ) ;
   }
   return ( NULL ) ;

} // end of MainWndProc()

//------------------------------------------------------------------------
//  BOOL FAR PASCAL AboutDlgProc
//
//  Description:
//     Dialog procedure for about box.
//
//  Parameters:
//
//  History:   Date       Author      Comment
//              6/ 5/93   BryanW      Wrote it.
//
//------------------------------------------------------------------------

BOOL FAR PASCAL AboutDlgProc
(
   HWND     hDlg,
   UINT     uMsg,
   WPARAM   wParam,
   LPARAM   lParam
)
{
   switch (uMsg)
   {
      case WM_INITDIALOG:
         return ( TRUE ) ;

      case WM_COMMAND:
         switch (wParam)
         {
            case IDOK:
            case IDCANCEL:
               EndDialog( hDlg, TRUE ) ;
               return ( TRUE ) ;
         }
         break ;
   }
   return ( FALSE ) ;

} // end of AboutDlgProc()

//------------------------------------------------------------------------
//  BOOL NEAR EnableNotification( HWND hWnd )
//
//  Description:
//     Enables notifications from DOSXFER.
//
//  Parameters:
//
//  History:   Date       Author      Comment
//              6/ 5/93   BryanW      Wrote it.
//
//------------------------------------------------------------------------

BOOL NEAR EnableNotification
(
   HWND  hWnd
)
{
   BOOL     fRetVal = FALSE ;
   DWORD    dwVxDEntry ;
   FARPROC  lpPostMessage ;
   HMODULE  hUser ;

   hUser = GetModuleHandle( "USER" ) ;
   lpPostMessage = GetProcAddress( (HINSTANCE) hUser,
                                   MAKEINTRESOURCE( 110 ) ) ;

   _asm
   {
      mov   ax, 0x1684
      mov   bx, DOSXFER_Device_ID
      int   0x2f
      mov   word ptr dwVxDEntry, di
      mov   word ptr dwVxDEntry + 2, es
      mov   ax, es
      or    ax, di
      jz    EN_Out

      mov   dx, 1                         ; DOSXFER_PM_Enable_CallBacks
      mov   ax, 1                         ; enable callbacks
      les   bx, lpPostMessage             ; ES:BX <- ptr of PostMessage
      mov   cx, hWnd                      ; CX gets hWnd

      call  dword ptr dwVxDEntry          ; call DOSXFER
      mov   fRetVal, TRUE

   EN_Out:

   }

   return ( fRetVal ) ;

} // end of EnableNotification()

//------------------------------------------------------------------------
//  BOOL GetDataFromDOSXFER
//
//  Description:
//
//
//  Parameters:
//
//  History:   Date       Author      Comment
//
//------------------------------------------------------------------------

BOOL NEAR GetDataFromDOSXFER
(
   LPNOTIFYSTRUCT lpNS,
   DWORD          dwData
)
{

   BOOL   fRetVal = FALSE ;
   DWORD  dwVxDEntry ;

   _asm
   {
      mov   ax, 0x1684
      mov   bx, DOSXFER_Device_ID
      int   0x2f
      mov   word ptr dwVxDEntry, di
      mov   word ptr dwVxDEntry + 2, es
      mov   ax, es
      or    ax, di
      jz    GD_Out

      push  ds

      mov   dx, 2                         ; DOSXFER_PM_Copy_Data
      mov   si, word ptr dwData           ; source node
      mov   bx, word ptr dwData + 2
      mov   di, word ptr lpNS             ; dest buffer
      mov   ds, word ptr lpNS + 2

      call  dword ptr dwVxDEntry          ; call DOSXFER

      pop   ds

      mov   fRetVal, TRUE

   GD_Out:

   }
   return ( fRetVal ) ;

} // end of GetDataFromDOSXFER()

//------------------------------------------------------------------------
//  BOOL DisableNotification
//
//  Description:
//
//
//  Parameters:
//
//  History:   Date       Author      Comment
//              6/ 5/93   BryanW      Wrote it.
//
//------------------------------------------------------------------------

BOOL NEAR DisableNotification()
{
   BOOL   fRetVal = FALSE ;
   DWORD  dwVxDEntry ;

   _asm
   {
      mov   ax, 0x1684
      mov   bx, DOSXFER_Device_ID
      int   0x2f
      mov   word ptr dwVxDEntry, di
      mov   word ptr dwVxDEntry + 2, es
      mov   ax, es
      or    ax, di
      jz    DN_Out

      mov   dx, 1                         ; DOSXFER_PM_Enable_CallBacks
      xor   ax, ax                        ; actually, we're disabling the
                                          ; notifications

      call  dword ptr dwVxDEntry          ; call DOSXFER

      mov   fRetVal, TRUE

   DN_Out:

   }
   return ( fRetVal ) ;

} // end of DisableNotification()

//---------------------------------------------------------------------------
//  End of File: winapp.c
//---------------------------------------------------------------------------

