        page    60, 132

;******************************************************************************
	title	VCONTEND - A simple virtual device driver example
;******************************************************************************
;
;   (C) Copyright Bryan A. Woodruff, 1993
;
;   Title:    VCONTEND.386 - Sample virtual device driver
;
;   Module:   VCONTEND.ASM - Core code
;
;   Version:  1.00
;
;   Date:     July 5, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    7/ 5/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a sample contention management scheme.
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE Shell.Inc
        INCLUDE Debug.Inc
        INCLUDE VContend.Inc
        .LIST

;==============================================================================
;           V I R T U A L   D E V I C E   D E C L A R A T I O N
;==============================================================================

Declare_Virtual_Device  VCONTEND, VCONTEND_Major_Ver, VCONTEND_Minor_Ver,\
                        VCONTEND_Control_Proc, VCONTEND_Device_ID,\
                        Undefined_Init_Order,,

;==============================================================================
;                          I N I T   D A T A
;==============================================================================

VxD_IDATA_SEG

Begin_VxD_IO_Table VCONTEND_Port_Table

        VxD_IO  TRAPIO_IDX,   VCONTEND_IO_Index_Reg
        VxD_IO  TRAPIO_DATA,  VCONTEND_IO_Data_Reg

End_VxD_IO_Table VCONTEND_Port_Table

VCONTEND_Port_Table_Entries equ (($-VCONTEND_Port_Table)-(SIZE VxD_IOT_Hdr)) / (SIZE VxD_IO_Struc)

VxD_IDATA_ENDS

;==============================================================================
;                  N O N - P A G E A B L E   D A T A
;==============================================================================

VxD_LOCKED_DATA_SEG

abData          db      (VCONTEND_Max_Index  + 1) dup (0)
hOwnerVM        dd      0
dwCBOffset      dd      0

bIndex          db      0

szContention    db      'VCONTEND.386 has detected a device contention.  '
                db      'The current VM can not use the hardware until it '
                db      'is released by the owning VM.', 0

VxD_LOCKED_DATA_ENDS

;==============================================================================
;                          I N I T   C O D E
;==============================================================================

VxD_ICODE_SEG

;------------------------------------------------------------------------------
;
;   VCONTEND_Device_Init
;
;   DESCRIPTION:
;       Non critical system initialization procedure.
;
;   ENTRY:
;       EBX = Sys VM handle
;
;   EXIT:
;       CLC if everything's A-OK, otherwise STC
;
;   USES:
;       Flags.
;
;------------------------------------------------------------------------------

BeginProc VCONTEND_Device_Init
   
        Trace_Out "VCONTEND: Device_Init"

        pushad

        VMMCall _Allocate_Device_CB_Area, <<size VCONTENDCB>, 0 >
        or      eax, eax
        jnz     SHORT DI_StoreCB
        stc
        jmp     SHORT DI_Exit

DI_StoreCB:
        mov     dwCBOffset, eax

        ;
        ; Build an I/O port table for Install_Mult_IO_Handlers
        ; using the base address.
        ;

        mov     ecx, VCONTEND_Port_Table_Entries
        mov     esi, OFFSET32 VCONTEND_Port_Table
        mov     edx, VCONTEND_Base_IO

DI_Install_IO_Handlers:
        mov     edi, esi                    ; save a copy in EDI
        add     esi, (size VxD_IOT_Hdr)

DI_Bump_IO_Loop:
        add     [esi.VxD_IO_Port], dx       ; add port base to offset
        add     esi, (size VxD_IO_Struc)
        loop    DI_Bump_IO_Loop

        ;
        ; Tell VMM to trap ports.
        ;

        VMMcall Install_Mult_IO_Handlers
ifdef DEBUG
        jnc     SHORT DI_Exit
        Debug_Out "VCONTEND: cannot trap ports!!"
endif

DI_Exit:
        popad
        ret

EndProc VCONTEND_Device_Init


VxD_ICODE_ENDS

VxD_LOCKED_CODE_SEG

;==============================================================================
;                      N O N P A G E A B L E   C O D E
;==============================================================================

;------------------------------------------------------------------------------
;
;   VCONTEND_Control_Proc
;
;   DESCRIPTION:
;       Dispatches VMM control messages to the appropriate handlers.
;
;   ENTRY:
;       EAX = Message
;       EBX = VM associated with message
;
;   EXIT:
;       Carry clear if no error (or if not handled by the VxD)
;       or set to indicate failure if the message can be failed.
;
;   USES:
;       All registers.
;
;------------------------------------------------------------------------------

BeginProc VCONTEND_Control_Proc

        Control_Dispatch Device_Init, VCONTEND_Device_Init
        Control_Dispatch VM_Not_Executeable, <SHORT VCONTEND_VM_Not_Executeable>

        clc
        ret

EndProc VCONTEND_Control_Proc

;------------------------------------------------------------------------------
;
;   VCONTNED_VM_Not_Executeable
;
;   DESCRIPTION:
;       This procedure checks if the VM being destroyed owns the virtual
;       hardware.  If so, then hOwnerVM is cleared.
;
;   ENTRY:
;       EBX = handle of VM being destroyed
;       EDX = Flags: VNE_Crashed, VNE_Nuked, VNE_CreateFail,
;                    VNE_CrInitFail, VNE_InitFail
;
;   EXIT:
;       Carry clear
;
;   USES:
;       FLAGS, EBX, EAX
;
;------------------------------------------------------------------------------

BeginProc VCONTEND_VM_Not_Executeable

        cmp     hOwnerVM, ebx
        jne     SHORT VNE_Exit
        mov     hOwnerVM, 0

VNE_Exit:
        clc
        ret

EndProc VCONTEND_VM_Not_Executeable

;------------------------------------------------------------------------------
;
;   VCONTEND_Check_Owner
;
;   DESCRIPTION:
;       Checks the current VM owner, if none, assigns
;       device to VM, otherwise if VM is owning VM, returns
;       carry clear otherwise carry set.
;
;   ENTRY:
;       EBX = VM Handle.
;
;   EXIT:
;       CLC if owner OK, or STC if contention
;
;   USES:
;       FLAGS
;
;------------------------------------------------------------------------------

BeginProc VCONTEND_Check_Owner, High_Freq

        push    eax
        mov     eax, hOwnerVM
        or      eax, eax
        jz      SHORT CO_Assign_To_VM
        cmp     eax, ebx
        jne     SHORT CO_Failure

CO_Success:
        pop     eax
        clc
        ret

CO_Assign_To_VM:
        mov     hOwnerVM, ebx
        jmp     SHORT CO_Success

CO_Failure:
        push    esi
        mov     esi, ebx
        add     esi, dwCBOffset

        test    [esi.dwFlags], F_ALREADYWARNED
        jnz     SHORT CO_SkipWarning

        ;
        ; Have the SHELL VxD put up an appopriate warning.
        ;

        pushad
        mov     eax, MB_OK or MB_ICONEXCLAMATION        ; message box flags
        mov     ecx, OFFSET32 szContention              ; message to display
        xor     esi, esi                                ; no callback
        xor     edi, edi                                ; default caption
        VxDcall SHELL_Message
        popad
        or      [esi.dwFlags], F_ALREADYWARNED

CO_SkipWarning:
        pop     esi
        pop     eax
        stc
        ret

EndProc VCONTEND_Check_Owner

;------------------------------------------------------------------------------
;
;   VCONTEND_IO_Index_Reg
;
;   DESCRIPTION:
;       Handles IO trapping.
;
;       This is a virtual R/W index register.
;
;   ENTRY:
;       EBX = VM Handle.
;       ECX = Type of I/O
;       EDX = Port number
;       EBP = Pointer to client register structure
;
;   EXIT:
;       EAX = data input or output depending on type of I/O
;
;   USES:
;       FLAGS
;
;------------------------------------------------------------------------------

BeginProc VCONTEND_IO_Index_Reg, High_Freq

        call    VCONTEND_Check_Owner
        jc      SHORT IIR_Exit

        Dispatch_Byte_IO Fall_Through, <SHORT IIR_Out>

        mov     al, bIndex
        clc
        ret

IIR_Out:
        cmp     al, VCONTEND_Max_Index
        ja      SHORT IIR_Exit
        mov     bIndex, al
        clc

IIR_Exit:
        ret

EndProc VCONTEND_IO_Index_Reg

;------------------------------------------------------------------------------
;
;   VCONTEND_IO_Data_Reg
;
;   DESCRIPTION:
;       Handles IO trapping.
;
;   ENTRY:
;       EBX = VM Handle.
;       ECX = Type of I/O
;       EDX = Port number
;       EBP = Pointer to client register structure
;
;   EXIT:
;       EAX = data input or output depending on type of I/O
;
;   USES:
;       FLAGS
;
;------------------------------------------------------------------------------

BeginProc VCONTEND_IO_Data_Reg, High_Freq

        call    VCONTEND_Check_Owner
        jc      SHORT IDR_Exit

        Dispatch_Byte_IO Fall_Through, <SHORT IDR_Out>

        push    ecx
        movzx   ecx, bIndex
        mov     al, abData[ ecx ]
        pop     ecx
        clc
        ret

IDR_Out:
        push    ecx
        movzx   ecx, bIndex
        mov     abData[ ecx ], al
        pop     ecx
        clc
        ret

IDR_Exit:
        ret

EndProc VCONTEND_IO_Data_Reg

VxD_LOCKED_CODE_ENDS

END

;---------------------------------------------------------------------------
;  End of File: vcontend.asm
;---------------------------------------------------------------------------

