        page    60, 132

;******************************************************************************
	title	VIRQD - Demonstrates IRQ virtualization
;******************************************************************************
;
;   (C) Copyright Bryan A. Woodruff, 1993
;
;   Title:    VIRQD.386 - Sample virtual device driver
;
;   Module:   VIRQD.ASM - Core code
;
;   Version:  1.00
;
;   Date:     July 5, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    7/ 5/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      A VxD demonstrating IRQ virtualization.
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE VPICD.Inc
        INCLUDE Debug.Inc
        .LIST

VIRQD_Major_Ver      equ     01h
VIRQD_Minor_Ver      equ     00h
VIRQD_Device_ID      equ     Undefined_Device_ID

VIRQD_Interrupt      equ     05h                ; grab an IRQ, any IRQ

;==============================================================================
;           V I R T U A L   D E V I C E   D E C L A R A T I O N
;==============================================================================

Declare_Virtual_Device  VIRQD, VIRQD_Major_Ver, VIRQD_Minor_Ver,\
                        VIRQD_Control_Proc, VIRQD_Device_ID,\
                        Undefined_Init_Order,,


;==============================================================================
;                      N O N - P A G E A B L E   D A T A
;==============================================================================

VxD_LOCKED_CODE_SEG

hVirtIRQ        dd      0

VxD_LOCKED_CODE_ENDS

;==============================================================================
;                             I N I T   D A T A
;==============================================================================

VxD_IDATA_SEG

VIRQD_IRQ_Descriptor VPICD_IRQ_Descriptor <,,                          \
                                           OFFSET32 VIRQD_Hw_Int_Proc,,\
                                           OFFSET32 VIRQD_EOI_Proc,,,>
VxD_IDATA_ENDS

;==============================================================================
;                             I N I T   C O D E
;==============================================================================

VxD_ICODE_SEG

;------------------------------------------------------------------------------
;
;   VIRQD_Device_Init
;
;   DESCRIPTION:
;       Non critical system initialization procedure.
;
;   ENTRY:
;       EBX = Sys VM handle
;
;   EXIT:
;       CLC if everything's A-OK, otherwise STC
;
;   USES:
;       Flags.
;
;------------------------------------------------------------------------------

BeginProc VIRQD_Device_Init
   
        Trace_Out "VIRQD: Device_Init"

        push    eax
        push    edi

        mov     edi, OFFSET32 VIRQD_IRQ_Descriptor
        mov     [edi.VID_IRQ_Number], VIRQD_Interrupt
        VxDCall VPICD_Virtualize_IRQ
ifdef DEBUG
        jnc     SHORT @F
        Debug_Out "VIRQD: Unable to virtualize IRQ"
        jmp     SHORT DI_Exit
@@:
else
        jc      SHORT DI_Exit
endif
        mov     hVirtIRQ, eax

DI_Exit:
        pop     edi
        pop     eax
        ret

EndProc VIRQD_Device_Init

VxD_ICODE_ENDS

VxD_LOCKED_CODE_SEG

;==============================================================================
;                      N O N P A G E A B L E   C O D E
;==============================================================================

;------------------------------------------------------------------------------
;
;   VIRQD_Control_Proc
;
;   DESCRIPTION:
;       Dispatches VMM control messages to the appropriate handlers.
;
;   ENTRY:
;       EAX = Message
;       EBX = VM associated with message
;
;   EXIT:
;       Carry clear if no error (or if not handled by the VxD)
;       or set to indicate failure if the message can be failed.
;
;   USES:
;       All registers.
;
;------------------------------------------------------------------------------

BeginProc VIRQD_Control_Proc

        Control_Dispatch Device_Init, VIRQD_Device_Init

        clc
        ret

EndProc VIRQD_Control_Proc

;==============================================================================
;            H A R D W A R E   I N T E R R U P T   P R O C E D U R E S
;==============================================================================

;------------------------------------------------------------------------------
;
;   VIRQD_Hw_Int_Proc
;
;   DESCRIPTION:
;       Hardware interrupt handler.  Called by VPICD.
;
;   ENTRY:
;       EAX = IRQ handle
;       EBX = current VM handle
;
;   EXIT:
;       CLC if processed, STC otherwise.
;
;   USES:
;       Flags.
;
;------------------------------------------------------------------------------

BeginProc VIRQD_Hw_Int_Proc, High_Freq

        Trace_Out "<i"

        VxDCall VPICD_Set_Int_Request

        clc
        ret

EndProc VIRQD_Hw_Int_Proc

;------------------------------------------------------------------------------
;
;   VIRQD_EOI_Proc
;
;   DESCRIPTION:
;       Hardware interrupt handler.  Called by VPICD.
;
;   ENTRY:
;       EAX = IRQ handle
;       EBX = current VM handle
;
;   EXIT:
;       CLC if processed, STC otherwise.
;
;   USES:
;       Flags.
;
;------------------------------------------------------------------------------

BeginProc VIRQD_EOI_Proc, High_Freq

        Trace_Out "i>"

        VxDCall VPICD_Clear_Int_Request
        VxDCall VPICD_Phys_EOI

        clc
        ret

EndProc VIRQD_EOI_Proc


VxD_LOCKED_CODE_ENDS

END

;---------------------------------------------------------------------------
;  End of File: virqd.asm
;---------------------------------------------------------------------------

