        page    60, 132

;******************************************************************************
	title	VSIMPLED - A simple virtual device driver example
;******************************************************************************
;
;   (C) Copyright Bryan A. Woodruff, 1993
;
;   Title:    VSIMPLED.386 - Sample virtual device driver
;
;   Module:   VSIMPLED.ASM - Core code
;
;   Version:  1.00
;
;   Date:     November 24, 1992
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;   11/24/92  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a minimal virtual device driver interface.
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE Debug.Inc
        .LIST

VSIMPLED_Major_Ver      equ     03h
VSIMPLED_Minor_Ver      equ     0Ah
VSIMPLED_Device_ID      equ     Undefined_Device_ID

;==============================================================================
;           V I R T U A L   D E V I C E   D E C L A R A T I O N
;==============================================================================

Declare_Virtual_Device  VSIMPLED, VSIMPLED_Major_Ver, VSIMPLED_Minor_Ver,\
                        VSIMPLED_Control_Proc, VSIMPLED_Device_ID,\
                        Undefined_Init_Order,,

;==============================================================================
;                             I C O D E
;==============================================================================

VxD_ICODE_SEG

;------------------------------------------------------------------------------
;
;   VSIMPLED_Sys_Critical_Init
;
;   Description:
;      On entry, interrupts are disabled.  Critical initialization
;      for this VxD should occur here.  For example, we can read
;      settings from VMM's cached copy of the SYSTEM.INI and act
;      set up our VxD as appropriate.
;
;      This procedure is called when the VSIMPLED_Control_Proc
;      dispatches the Sys_Critical_Init notification from VMM.
;
;      We can notify VMM of failure by returning with carry set
;      or carry clear will suggest success.
;
;------------------------------------------------------------------------------

BeginProc VSIMPLED_Sys_Critical_Init

        Trace_Out "VSIMPLED: Sys_Critical_Init"

        clc
        ret

EndProc VSIMPLED_Sys_Critical_Init

;------------------------------------------------------------------------------
;
;   VSIMPLED_Device_Init
;
;   Description:
;      This is a non-system critical initialization procedure.
;      IRQ virtualization, I/O port trapping and VM control
;      block allocation can occur here.
;
;      Again, the same return value applies...
;      CLC for success, STC for error notification.
;
;------------------------------------------------------------------------------

BeginProc VSIMPLED_Device_Init
   
        Trace_Out "VSIMPLED: Device_Init"

        clc
        ret

EndProc VSIMPLED_Device_Init


VxD_ICODE_ENDS

VxD_LOCKED_CODE_SEG

;==============================================================================
;                      N O N P A G E A B L E   C O D E
;==============================================================================

;------------------------------------------------------------------------------
;
;   VSIMPLED_Control_Proc
;
;   DESCRIPTION:
;       Dispatches VMM control messages to the appropriate handlers.
;
;   ENTRY:
;       EAX = Message
;       EBX = VM associated with message
;
;   EXIT:
;       Carry clear if no error (or if not handled by the VxD)
;       or set to indicate failure if the message can be failed.
;
;   USES:
;       All registers.
;
;------------------------------------------------------------------------------

BeginProc VSIMPLED_Control_Proc

        Control_Dispatch Sys_Critical_Init, VSIMPLED_Sys_Critical_Init
        Control_Dispatch Device_Init, VSIMPLED_Device_Init

        clc
        ret

EndProc VSIMPLED_Control_Proc

VxD_LOCKED_CODE_ENDS

END

;---------------------------------------------------------------------------
;  End of File: vsimpled.asm
;---------------------------------------------------------------------------

