        page    60, 132

;******************************************************************************
	title	VTRAPIOD - A simple virtual device driver example
;******************************************************************************
;
;   (C) Copyright Bryan A. Woodruff, 1993
;
;   Title:    VTRAPIOD.386 - Sample virtual device driver
;
;   Module:   VTRAPIOD.ASM - Core code
;
;   Version:  1.00
;
;   Date:     July 5, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    7/ 5/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      A sample I/O trapping virtual device.  This virtualizes
;      a siomple hardware interface at 350h-351h.
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE Debug.Inc
        .LIST

VTRAPIOD_Major_Ver      equ     01h
VTRAPIOD_Minor_Ver      equ     00h
VTRAPIOD_Device_ID      equ     Undefined_Device_ID

VTRAPIOD_Base_IO        equ     350h
TRAPIO_IDX              equ     00h
TRAPIO_DATA             equ     01h

;==============================================================================
;           V I R T U A L   D E V I C E   D E C L A R A T I O N
;==============================================================================

Declare_Virtual_Device  VTRAPIOD, VTRAPIOD_Major_Ver, VTRAPIOD_Minor_Ver,\
                        VTRAPIOD_Control_Proc, VTRAPIOD_Device_ID,\
                        Undefined_Init_Order,,

;==============================================================================
;                          I N I T   D A T A
;==============================================================================

VxD_IDATA_SEG

Begin_VxD_IO_Table VTRAPIOD_Port_Table

        VxD_IO  TRAPIO_IDX,   VTRAPIOD_IO_Index_Reg
        VxD_IO  TRAPIO_DATA,  VTRAPIOD_IO_Data_Reg

End_VxD_IO_Table VTRAPIOD_Port_Table

VTRAPIOD_Port_Table_Entries equ (($-VTRAPIOD_Port_Table)-(SIZE VxD_IOT_Hdr)) / (SIZE VxD_IO_Struc)

VxD_IDATA_ENDS

;==============================================================================
;                  N O N - P A G E A B L E   D A T A
;==============================================================================

VxD_LOCKED_DATA_SEG

bIndex  db      0

VTRAPIOD_Max_Index equ 0Fh

abData  db      VTRAPIOD_Max_Index dup (0)

VxD_LOCKED_DATA_ENDS

;==============================================================================
;                          I N I T   C O D E
;==============================================================================

VxD_ICODE_SEG

;------------------------------------------------------------------------------
;
;   VTRAPIOD_Device_Init
;
;   DESCRIPTION:
;       Non critical system initialization procedure.
;
;   ENTRY:
;       EBX = Sys VM handle
;
;   EXIT:
;       CLC if everything's A-OK, otherwise STC
;
;   USES:
;       Flags.
;
;------------------------------------------------------------------------------

BeginProc VTRAPIOD_Device_Init
   
        Trace_Out "VTRAPIOD: Device_Init"

        pushad

        ;
        ; Build an I/O port table for Install_Mult_IO_Handlers
        ; using the base address.
        ;

        mov     ecx, VTRAPIOD_Port_Table_Entries
        mov     esi, OFFSET32 VTRAPIOD_Port_Table
        mov     edx, VTRAPIOD_Base_IO

DI_Install_IO_Handlers:
        mov     edi, esi                    ; save a copy in EDI
        add     esi, (size VxD_IOT_Hdr)

DI_Bump_IO_Loop:
        add     [esi.VxD_IO_Port], dx       ; add port base to offset
        add     esi, (size VxD_IO_Struc)
        loop    DI_Bump_IO_Loop

        ;
        ; Tell VMM to trap ports.
        ;

        VMMcall Install_Mult_IO_Handlers
ifdef DEBUG
        jnc     SHORT DI_Exit
        Debug_Out "VTRAPIOD: cannot trap ports!!"
endif

DI_Exit:
        popad
        ret

EndProc VTRAPIOD_Device_Init


VxD_ICODE_ENDS

VxD_LOCKED_CODE_SEG

;==============================================================================
;                      N O N P A G E A B L E   C O D E
;==============================================================================

;------------------------------------------------------------------------------
;
;   VTRAPIOD_Control_Proc
;
;   DESCRIPTION:
;       Dispatches VMM control messages to the appropriate handlers.
;
;   ENTRY:
;       EAX = Message
;       EBX = VM associated with message
;
;   EXIT:
;       Carry clear if no error (or if not handled by the VxD)
;       or set to indicate failure if the message can be failed.
;
;   USES:
;       All registers.
;
;------------------------------------------------------------------------------

BeginProc VTRAPIOD_Control_Proc

        Control_Dispatch Device_Init, VTRAPIOD_Device_Init

        clc
        ret

EndProc VTRAPIOD_Control_Proc

;------------------------------------------------------------------------------
;
;   VTRAPIOD_IO_Index_Reg
;
;   DESCRIPTION:
;       Handles IO trapping.
;
;       This is a virtual R/W index register.
;
;   ENTRY:
;       EBX = VM Handle.
;       ECX = Type of I/O
;       EDX = Port number
;       EBP = Pointer to client register structure
;
;   EXIT:
;       EAX = data input or output depending on type of I/O
;
;   USES:
;       FLAGS
;
;------------------------------------------------------------------------------

BeginProc VTRAPIOD_IO_Index_Reg, High_Freq

        Dispatch_Byte_IO Fall_Through, <SHORT IIR_Out>

        mov     al, bIndex
        clc
        ret

IIR_Out:
        cmp     al, VTRAPIOD_Max_Index
        ja      SHORT IIR_Exit
        mov     bIndex, al

IIR_Exit:
        clc
        ret

EndProc VTRAPIOD_IO_Index_Reg

;------------------------------------------------------------------------------
;
;   VTRAPIOD_IO_Data_Reg
;
;   DESCRIPTION:
;       Handles IO trapping.
;
;   ENTRY:
;       EBX = VM Handle.
;       ECX = Type of I/O
;       EDX = Port number
;       EBP = Pointer to client register structure
;
;   EXIT:
;       EAX = data input or output depending on type of I/O
;
;   USES:
;       FLAGS
;
;------------------------------------------------------------------------------

BeginProc VTRAPIOD_IO_Data_Reg, High_Freq

        Dispatch_Byte_IO Fall_Through, <SHORT IDR_Out>

        push    ecx
        movzx   ecx, bIndex
        mov     al, abData[ ecx ]
        pop     ecx
        clc
        ret

IDR_Out:
        push    ecx
        movzx   ecx, bIndex
        mov     abData[ ecx ], al
        pop     ecx
        clc
        ret

EndProc VTRAPIOD_IO_Data_Reg

VxD_LOCKED_CODE_ENDS

END

;---------------------------------------------------------------------------
;  End of File: vtrapiod.asm
;---------------------------------------------------------------------------

