        page    60, 132

;******************************************************************************
        title   VDDINTA - assembly routines for VDDINT.C
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VDDINTA - assembly routines for VDDINT.C
;
;   Module:   VDDINTA.ASM
;
;   Version:  1.00
;
;   Date:     October 25, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------  ------------------------------------------  -----------
;   10/25/93  1.00      Pulled from original DDK sources            BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      V86 Int 10h hack-o-matic.  This code is best left to assembly.
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

.xlist
        INCLUDE VMM.INC
        INCLUDE DEBUG.INC
.list

;==============================================================================
;                                 D A T A
;==============================================================================

VxD_DATA_SEG

        ;
        ; INT 10h real mode seg:offset hook vectors
        ;

	PUBLIC	Vid_I10_OldVector
Vid_I10_OldVector   DD	    0			; Previous INT 10h vector
	PUBLIC	Vid_Stub_I10_Vector
Vid_Stub_I10_Vector DD	    ?			; Address of our INT 10h routine

VxD_DATA_ENDS

;==============================================================================
;                   E X T E R N A L   R E F E R E N C E S
;==============================================================================

VxD_CODE_SEG

   EXTRN  VDD_Int10_ModeSwitch:NEAR
   EXTRN  VDD_Int10_ModeSwDone:NEAR

VxD_CODE_ENDS
;==============================================================================
;                               I C O D E
;==============================================================================

VxD_ICODE_SEG

;----------------------------------------------------------------------------
;
;   VDD_Int_10_Stub
;
;   DESCRIPTION:
;      Stub of code which is copied down into V86 memory and is
;      hooked into the INT 10 chain so that VDD can be notified
;      of mode change INT 10's.  Note that this is real mode 16
;      bit code but since it is in a 32 bit segment as far as the
;      assembler is concerned, we specify everything as DWORD.
;      Do not use any protect mode instructions herein.
;
;   ENTRY:
;      none - this routine is not to be called in pmode!!!!!!!!!
;
;   EXIT:
;
;   USES:
;
;----------------------------------------------------------------------------

; don't use BeginProc/EndProc to avoid any
; preamble stuff!

VDD_Int_10_Stub PROC NEAR    
                             
        or      ah, ah                ;Q: Mode setting?
        je      SHORT VIS_mode        ;   Y: notify VDD and reflect
        cmp     ah,05h                ;Q: Page changeing?
        jz      SHORT VIS_mode        ;   Y: notify VDD and reflect
        cmp     ah, 11h               ;Q: Mode setting(via char set chg)?
        jne     SHORT VIS_reflect     ;   N: reflect
        cmp     al, 3                 ;Q: Set char map
        je      SHORT VIS_reflect     ;   Y: reflect
        cmp     al, 30h               ;Q: Get font info?
        je      SHORT VIS_reflect     ;   Y: reflect
VIS_mode:
VIS_start_bp equ $ - VDD_Int_10_Stub
        nop                           ; a V86 brkpt will be installed here

        pushfd                        ; in 16 bit segment this is a pushf
        ;call old handler
        db      9Ah
VIS_call equ $ - VDD_Int_10_Stub
        dd      ?

VIS_end_bp equ $ - VDD_Int_10_Stub
        nop                           ; a V86 brkpt will be installed here
        iretd                         ; in 16 bit segment this is an iret
VIS_reflect:

        ;jmp to old handler
        db      0EAh
VIS_jmp equ $ - VDD_Int_10_Stub
        dd      ?

VIS_stub_size equ $ - VDD_Int_10_Stub

VDD_Int_10_Stub ENDP

;----------------------------------------------------------------------------
;
;   VOID PASCAL VDD_Int_MungeInt10h( DWORD hSysVM )
;
;   DESCRIPTION:
;       Set INT 10h stub.
;
;   ENTRY:
;       hSysVM
;          handle to system VM
;
;   EXIT:
;       None
;
;   USES:
;       Flags
;
;----------------------------------------------------------------------------

BeginProc VDD_Int_MungeInt10h

        hVM     equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        pushad

        mov     ebx, hVM

        ;
        ; Copy Int 10h stub into V86 memory and save old int vector
        ;

        mov     eax, 10h
        VMMCall Get_V86_Int_Vector
        shl     ecx, 16
        mov     cx, dx
        mov     [Vid_I10_OldVector], ecx

        push    ecx
        VMMCall _Allocate_Global_V86_Data_Area, <VIS_stub_size, 0>
        pop     edx
        or      eax, eax
        jnz     SHORT alloc_ok

Debug_Out "ERROR: Could not allocate global VM data area for VDD int 10 hook"

        VMMCall Fatal_Memory_Error

alloc_ok:
        mov     edi, eax
        mov     esi, OFFSET32 VDD_Int_10_Stub
        mov     ecx, VIS_stub_size
        cld
        Begin_Touch_1st_Meg
        rep     movsb
        mov     edi, eax
        mov     [edi+VIS_call], edx         ; patch CALL to point to old handler
        mov     [edi+VIS_jmp], edx          ; patch JMP to point to old handler
        End_Touch_1st_Meg
        shr     eax, 4
        movzx   ecx, ax
        mov     edx, edi
        and     edx, 0Fh
        mov     eax, 10h
        VMMCall Set_V86_Int_Vector
        mov     eax, ecx
        shl     eax, 16
        add     eax, edx
        mov     [Vid_Stub_I10_Vector], eax  ; save our new vector setting
        add     eax, VIS_start_bp
        xor     edx, edx
        mov     esi, OFFSET32 VDD_Int10_ModeSwitch
        VMMCall Install_V86_Break_Point
        add     eax, VIS_end_bp - VIS_start_bp
        mov     esi, OFFSET32 VDD_Int10_ModeSwDone
        VMMCall Install_V86_Break_Point

        popad

        pop     ebp
        ret     4

EndProc VDD_Int_MungeInt10h

VxD_ICODE_ENDS

end
