//---------------------------------------------------------------------------
//
//  Module: vxdinit.c
//
//  Purpose:
//     Init code and data for VGLOBALD.
//
//  Development Team:
//     Bryan A. Woodruff
//
//  History:   Date       Author      Comment
//              6/13/93   BryanW      Wrote it.
//
//---------------------------------------------------------------------------
//
//               Copyright (c) 1993 Woodruff Software Systems.
//                          All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <vmm.h>
#include "vglobald.h"

#pragma data_seg( "_IDATA", "ICODE" ) ;

//=============================================================================
//                             I C O D E
//=============================================================================

//------------------------------------------------------------------------
//  BOOL VGLOBALD_Device_Init
//
//  Description:
//     This is a non-system critical initialization procedure.
//     IRQ virtualization, I/O port trapping and VM control
//     block allocation can occur here.
//
//     Again, the same return value applies... TRUE for success,
//     FALSE for error notification.
//
//  Parameters:
//     hVM
//        System VM handle
//
//     pCmdTail
//        pointer to WIN.COM's command tail
//
//     pCRS
//        pointer to System VM client register structure
//
//  History:   Date       Author      Comment
//              3/ 9/93   BryanW      Wrote it.
//
//------------------------------------------------------------------------

BOOL CDECL VGLOBALD_Device_Init
(
   DWORD     hVM,
   PSTR      pCmdTail,
   PCRS_32   pCRS
)
{
   DWORD  dwGlobalArea ;
   PVMCB  pSysVMCB ;

   UNUSED_PARAM( pCRS ) ;
   UNUSED_PARAM( pCmdTail ) ;

   vmmTraceOut( "VGLOBALD_Device_Init\r\n" ) ;

   // Allocate the VM control block area

   if (!(guCBOffset = vmmAllocateDeviceCBArea( sizeof( VMCB ), NULL )))
   {
      vmmDebugOut( "Failed to allocate control block!\r\n" ) ;
      return ( FALSE ) ;
   }

   pSysVMCB = (PVMCB) (hVM + guCBOffset) ;

   // Allocate a global V86 data area of 1 page

   if (NULL ==
         (dwGlobalArea =
            vmmAllocateGlobalV86DataArea( P_SIZE, GVDAPageAlign )))
   {
      vmmDebugOut( "Failed to allocate global V86 data area!\r\n" ) ;
      return ( FALSE ) ;
   }

   vmmTraceOutParam( "Allocated global area at #EAX\r\n", dwGlobalArea ) ;

   gwGlobalPage = dwGlobalArea >> 12 ;

   gpPageDir = (PDWORD) vmmMapPhysToLinear( GetCR3(), P_SIZE, NULL ) ;

   vmmTraceOutParam( "CR3 (linear) = #eax\r\n", (DWORD) gpPageDir ) ;

   if (NULL == (pVGLOBALD_Invalid_PFault =
                  vmmwrapThunkIPFProc( VGLOBALD_Invalid_PFault )))
   {
      vmmDebugOut( "Failure: Could not allocate Page_Fault thunk!\r\n" ) ;
      return ( FALSE ) ;
   }

   vmmHookInvalidPageFault( pVGLOBALD_Invalid_PFault ) ;

   // Setup page table pointer for SysVM

   pSysVMCB -> pPageTable =
      (PDWORD) vmmMapPhysToLinear( gpPageDir[ 0 ] & 0xFFFFF000,
                                   P_SIZE, NULL ) ;

   ghOwnerVM = hVM ;

   return ( TRUE ) ;

} // end of VGLOBALD_Device_Init()

//---------------------------------------------------------------------------
//  End of File: vxdinit.c
//---------------------------------------------------------------------------
