//---------------------------------------------------------------------------
//
//  Module: vmm.h
//
//  Purpose:
//     Provides necessary macros for the 'C' wrapper.
//
//  Development Team:
//     Bryan A. Woodruff
//
//  History:   Date       Author      Comment
//              3/ 8/93   BryanW      Wrote it.
//
//---------------------------------------------------------------------------
//
//               Copyright (c) 1993 Woodruff Software Systems.
//                          All Rights Reserved.
//
//---------------------------------------------------------------------------

#define VOID               void

#define PASCAL             _pascal
#define CDECL              _cdecl

typedef int                BOOL;
#define FALSE              0
#define TRUE               (~FALSE)

typedef unsigned char      BYTE;
typedef unsigned short     WORD;
typedef unsigned long      DWORD;

typedef unsigned int       UINT;

#ifndef NULL
#define NULL               0
#endif

typedef char *             PSTR;
typedef BYTE *             PBYTE;
typedef int *              PINT;
typedef UINT *             PUINT;
typedef WORD *             PWORD;
typedef DWORD *            PDWORD;

#define LONG               long

#define LOBYTE(w)          ((BYTE)(w))
#define HIBYTE(w)          ((BYTE)(((UINT)(w) >> 8) & 0xFF))
                            
#define LOWORD(l)          ((WORD)(DWORD)(l))
#define HIWORD(l)          ((WORD)((((DWORD)(l)) >> 16) & 0xFFFF))

#define MAKELONG(low, high) ((LONG)(((WORD)(low)) | (((DWORD)((WORD)(high))) << 16)))

//----------------------------------------------------------------------------
//
//                        Device ID equates
//
//----------------------------------------------------------------------------

#define Undefined_Device_ID    0x00000
#define VMM_Device_ID          0x00001   // Used for dynalink table
#define Debug_Device_ID        0x00002
#define VPICD_Device_ID        0x00003
#define VDMAD_Device_ID        0x00004
#define VTD_Device_ID          0x00005
#define V86MMGR_Device_ID      0x00006
#define PageSwap_Device_ID     0x00007
#define Parity_Device_ID       0x00008
#define Reboot_Device_ID       0x00009
#define VDD_Device_ID          0x0000A
#define VSD_Device_ID          0x0000B
#define VMD_Device_ID          0x0000C
#define VKD_Device_ID          0x0000D
#define VCD_Device_ID          0x0000E
#define VPD_Device_ID          0x0000F
#define BlockDev_Device_ID     0x00010
#define VMCPD_Device_ID        0x00011
#define EBIOS_Device_ID        0x00012
#define BIOSXlat_Device_ID     0x00013
#define VNETBIOS_Device_ID     0x00014
#define DOSMGR_Device_ID       0x00015
#define WINLOAD_Device_ID      0x00016
#define SHELL_Device_ID        0x00017
#define VMPoll_Device_ID       0x00018
#define VPROD_Device_ID        0x00019
#define DOSNET_Device_ID       0x0001A
#define VFD_Device_ID          0x0001B
#define VDD2_Device_ID         0x0001C   // Secondary display adapter
#define WINDEBUG_Device_ID     0x0001D
#define TSRLoad_Device_ID      0x0001E   // TSR instance utility ID
#define BiosHook_Device_ID     0x0001F   // Bios interrupt hooker VxD
#define Int13_Device_ID        0x00020
#define PageFile_Device_ID     0x00021   // Paging File device
#define SCSI_Device_ID         0x00022   // SCSI device
#define MCA_POS_Device_ID      0x00023   // MCA_POS device
#define SCSIFD_Device_ID       0x00024   // SCSI FastDisk device
#define VPEND_Device_ID        0x00025   // Pen device
#define APM_Device_ID          0x00026   // Power Management device

//----------------------------------------------------------------------------
//
//                     Initialization order equates
//
//----------------------------------------------------------------------------

#define VMM_Init_Order         0x000000000
#define APM_Init_Order         0x001000000
#define Debug_Init_Order       0x004000000
#define BiosHook_Init_Order    0x006000000
#define VPROD_Init_Order       0x008000000
#define VPICD_Init_Order       0x00C000000
#define VTD_Init_Order         0x014000000
#define PageFile_Init_Order    0x018000000
#define PageSwap_Init_Order    0x01C000000
#define Parity_Init_Order      0x020000000
#define Reboot_Init_Order      0x024000000
#define EBIOS_Init_Order       0x026000000
#define VDD_Init_Order         0x028000000
#define VSD_Init_Order         0x02C000000
#define VCD_Init_Order         0x030000000
#define VMD_Init_Order         0x034000000
#define VKD_Init_Order         0x038000000
#define VPD_Init_Order         0x03C000000
#define BlockDev_Init_Order    0x040000000
#define MCA_POS_Init_Order     0x041000000
#define SCSIFD_Init_Order      0x041400000
#define SCSIMaster_Init_Order  0x041800000
#define Int13_Init_Order       0x042000000
#define VFD_Init_Order         0x044000000
#define VMCPD_Init_Order       0x048000000
#define BIOSXlat_Init_Order    0x050000000
#define VNETBIOS_Init_Order    0x054000000
#define DOSMGR_Init_Order      0x058000000
#define DOSNET_Init_Order      0x05C000000
#define WINLOAD_Init_Order     0x060000000
#define VMPoll_Init_Order      0x064000000

#define Undefined_Init_Order   0x080000000

#define WINDEBUG_Init_Order    0x081000000
#define VDMAD_Init_Order       0x090000000
#define V86MMGR_Init_Order     0x0A0000000

// Device ID for device that must touch memory in 1st MB
// at crit init (after V86mmgr)

#define Undef_Touch_Mem_Init_Order 0x0A8000000
#define SHELL_Init_Order           0x0B0000000

//----------------------------------------------------------------------------
//
//                      System control call messages
//
//----------------------------------------------------------------------------
                            
#define Sys_Critical_Init     0x0000
#define Device_Init           0x0001
#define Init_Complete         0x0002
#define Sys_VM_Init           0x0003
#define Sys_VM_Terminate      0x0004
#define System_Exit           0x0005
#define Sys_Critical_Exit     0x0006
#define Create_VM             0x0007
#define VM_Critical_Init      0x0008
#define VM_Init               0x0009
#define VM_Terminate          0x000A
#define VM_Not_Executable     0x000B
#define Destroy_VM            0x000C
#define VM_Suspend            0x000D
#define VM_Resume             0x000E
#define Set_Device_Focus      0x000F
#define Begin_Message_Mode    0x0010
#define End_Message_Mode      0x0011

#define Reboot_Processor      0x0012
#define Query_Destroy         0x0013
#define Debug_Query           0x0014

#define Begin_PM_App          0x0015

#define BPA_32_Bit            0x0001
#define BPA_32_Bit_Flag       0x0001

#define End_PM_App            0x0016

#define Device_Reboot_Notify  0x0017
#define Crit_Reboot_Notify    0x0018

#define Close_VM_Notify       0x0019

#define CVNF_Crit_Close       0x0001
#define CNVF_Crit_Close_Bit   0x0000

#define Power_Event           0x001A

//----------------------------------------------------------------------------
//
//                          VMM Service Definitions
//
//----------------------------------------------------------------------------

#define Get_Cur_VM_Handle                   0x00010001
#define Test_Cur_VM_Handle                  0x00010002
#define Get_Sys_VM_Handle                   0x00010003
#define Test_Sys_VM_Handle                  0x00010004
#define Validate_VM_Handle                  0x00010005

#define Get_VMM_Reenter_Count               0x00010006
#define Begin_Reentrant_Execution           0x00010007
#define End_Reentrant_Execution             0x00010008

#define Install_V86_Break_Point             0x00010009
#define Remove_V86_Break_Point              0x0001000a
#define Allocate_V86_Call_Back              0x0001000b
#define Allocate_PM_Call_Back               0x0001000c

#define Call_When_VM_Returns                0x0001000d

#define Schedule_Global_Event               0x0001000e
#define Schedule_VM_Event                   0x0001000f
#define Call_Global_Event                   0x00010010
#define Call_VM_Event                       0x00010011
#define Cancel_Global_Event                 0x00010012
#define Cancel_VM_Event                     0x00010013
#define Call_Priority_VM_Event              0x00010014
#define Cancel_Priority_VM_Event            0x00010015

#define Get_NMI_Handler_Addr                0x00010016
#define Set_NMI_Handler_Addr                0x00010017
#define Hook_NMI_Event                      0x00010018

#define Call_When_VM_Ints_Enabled           0x00010019
#define Enable_VM_Ints                      0x0001001a
#define Disable_VM_Ints                     0x0001001b

#define Map_Flat                            0x0001001c
#define Map_Lin_To_VM_Addr                  0x0001001d

#define Adjust_Exec_Priority                0x0001001e
#define Begin_Critical_Section              0x0001001f
#define End_Critical_Section                0x00010020
#define End_Crit_And_Suspend                0x00010021
#define Claim_Critical_Section              0x00010022
#define Release_Critical_Section            0x00010023
#define Call_When_Not_Critical              0x00010024
#define Create_Semaphore                    0x00010025
#define Destroy_Semaphore                   0x00010026
#define Wait_Semaphore                      0x00010027
#define Signal_Semaphore                    0x00010028
#define Get_Crit_Section_Status             0x00010029
#define Call_When_Task_Switched             0x0001002a
#define Suspend_VM                          0x0001002b
#define Resume_VM                           0x0001002c
#define No_Fail_Resume_VM                   0x0001002d
#define Nuke_VM                             0x0001002e
#define Crash_Cur_VM                        0x0001002f

#define Get_Execution_Focus                 0x00010030
#define Set_Execution_Focus                 0x00010031
#define Get_Time_Slice_Priority             0x00010032
#define Set_Time_Slice_Priority             0x00010033
#define Get_Time_Slice_Granularity          0x00010034
#define Set_Time_Slice_Granularity          0x00010035
#define Get_Time_Slice_Info                 0x00010036
#define Adjust_Execution_Time               0x00010037
#define Release_Time_Slice                  0x00010038
#define Wake_Up_VM                          0x00010039
#define Call_When_Idle                      0x0001003a

#define Get_Next_VM_Handle                  0x0001003b

#define Set_Global_Time_Out                 0x0001003c
#define Set_VM_Time_Out                     0x0001003d
#define Cancel_Time_Out                     0x0001003e
#define Get_System_Time                     0x0001003f
#define Get_VM_Exec_Time                    0x00010040

#define Hook_V86_Int_Chain                  0x00010041
#define Get_V86_Int_Vector                  0x00010042
#define Set_V86_Int_Vector                  0x00010043
#define Get_PM_Int_Vector                   0x00010044
#define Set_PM_Int_Vector                   0x00010045

#define Simulate_Int                        0x00010046
#define Simulate_Iret                       0x00010047
#define Simulate_Far_Call                   0x00010048
#define Simulate_Far_Jmp                    0x00010049
#define Simulate_Far_Ret                    0x0001004a
#define Simulate_Far_Ret_N                  0x0001004b
#define Build_Int_Stack_Frame               0x0001004c

#define Simulate_Push                       0x0001004d
#define Simulate_Pop                        0x0001004e

#define _HeapAllocate                       0x0001004f
#define _HeapReAllocate                     0x00010050
#define _HeapFree                           0x00010051
#define _HeapGetSize                        0x00010052

#define _PageAllocate                       0x00010053
#define _PageReAllocate                     0x00010054
#define _PageFree                           0x00010055
#define _PageLock                           0x00010056
#define _PageUnLock                         0x00010057
#define _PageGetSizeAddr                    0x00010058
#define _PageGetAllocInfo                   0x00010059
#define _GetFreePageCount                   0x0001005a
#define _GetSysPageCount                    0x0001005b
#define _GetVMPgCount                       0x0001005c
#define _MapIntoV86                         0x0001005d
#define _PhysIntoV86                        0x0001005e
#define _TestGlobalV86Mem                   0x0001005f
#define _ModifyPageBits                     0x00010060
#define _CopyPageTable                      0x00010061
#define _LinMapIntoV86                      0x00010062
#define _LinPageLock                        0x00010063
#define _LinPageUnLock                      0x00010064
#define _SetResetV86Pageable                0x00010065
#define _GetV86PageableArray                0x00010066
#define _PageCheckLinRange                  0x00010067
#define _PageOutDirtyPages                  0x00010068
#define _PageDiscardPages                   0x00010069

#define _GetNulPageHandle                   0x0001006a
#define _GetFirstV86Page                    0x0001006b
#define _MapPhysToLinear                    0x0001006c
#define _GetAppFlatDSAlias                  0x0001006d
#define _SelectorMapFlat                    0x0001006e
#define _GetDemandPageInfo                  0x0001006f

#define _GetSetPageOutCount                 0x00010070

#define Hook_V86_Page                       0x00010071
#define _Assign_Device_V86_Pages            0x00010072
#define _DeAssign_Device_V86_Pages          0x00010073
#define _Get_Device_V86_Pages_Array         0x00010074
#define MMGR_SetNULPageAddr                 0x00010075

#define _Allocate_GDT_Selector              0x00010076
#define _Free_GDT_Selector                  0x00010077
#define _Allocate_LDT_Selector              0x00010078
#define _Free_LDT_Selector                  0x00010079
#define _BuildDescriptorDWORDs              0x0001007a
#define _GetDescriptor                      0x0001007b
#define _SetDescriptor                      0x0001007c
#define _MMGR_Toggle_HMA                    0x0001007d
#define Get_Fault_Hook_Addrs                0x0001007e
#define Hook_V86_Fault                      0x0001007f
#define Hook_PM_Fault                       0x00010080
#define Hook_VMM_Fault                      0x00010081
#define Begin_Nest_V86_Exec                 0x00010082
#define Begin_Nest_Exec                     0x00010083
#define Exec_Int                            0x00010084
#define Resume_Exec                         0x00010085
#define End_Nest_Exec                       0x00010086
#define Allocate_PM_App_CB_Area             0x00010087
#define Get_Cur_PM_App_CB                   0x00010088
#define Set_V86_Exec_Mode                   0x00010089
#define Set_PM_Exec_Mode                    0x0001008a
#define Begin_Use_Locked_PM_Stack           0x0001008b
#define End_Use_Locked_PM_Stack             0x0001008c
#define Save_Client_State                   0x0001008d
#define Restore_Client_State                0x0001008e
#define Exec_VxD_Int                        0x0001008f
#define Hook_Device_Service                 0x00010090
#define Hook_Device_V86_API                 0x00010091
#define Hook_Device_PM_API                  0x00010092
#define System_Control                      0x00010093
#define Simulate_IO                         0x00010094
#define Install_Mult_IO_Handlers            0x00010095
#define Install_IO_Handler                  0x00010096
#define Enable_Global_Trapping              0x00010097
#define Enable_Local_Trapping               0x00010098
#define Disable_Global_Trapping             0x00010099
#define Disable_Local_Trapping              0x0001009a
#define List_Create                         0x0001009b
#define List_Destroy                        0x0001009c
#define List_Allocate                       0x0001009d
#define List_Attach                         0x0001009e
#define List_Attach_Tail                    0x0001009f
#define List_Insert                         0x000100a0
#define List_Remove                         0x000100a1
#define List_Deallocate                     0x000100a2
#define List_Get_First                      0x000100a3
#define List_Get_Next                       0x000100a4
#define List_Remove_First                   0x000100a5
#define _AddInstanceItem                    0x000100a6
#define _Allocate_Device_CB_Area            0x000100a7
#define _Allocate_Global_V86_Data_Area      0x000100a8
#define _Allocate_Temp_V86_Data_Area        0x000100a9
#define _Free_Temp_V86_Data_Area            0x000100aa
#define Get_Profile_Decimal_Int             0x000100ab
#define Convert_Decimal_String              0x000100ac
#define Get_Profile_Fixed_Point             0x000100ad
#define Convert_Fixed_Point_String          0x000100ae
#define Get_Profile_Hex_Int                 0x000100af
#define Convert_Hex_String                  0x000100b0
#define Get_Profile_Boolean                 0x000100b1
#define Convert_Boolean_String              0x000100b2
#define Get_Profile_String                  0x000100b3
#define Get_Next_Profile_String             0x000100b4
#define Get_Environment_String              0x000100b5
#define Get_Exec_Path                       0x000100b6
#define Get_Config_Directory                0x000100b7
#define OpenFile                            0x000100b8
#define Get_PSP_Segment                     0x000100b9
#define GetDOSVectors                       0x000100ba
#define Get_Machine_Info                    0x000100bb
#define GetSet_HMA_Info                     0x000100bc
#define Set_System_Exit_Code                0x000100bd
#define Fatal_Error_Handler                 0x000100be
#define Fatal_Memory_Error                  0x000100bf
#define Update_System_Clock                 0x000100c0
#define Test_Debug_Installed		           0x000100c1
#define Out_Debug_String		              0x000100c2
#define Out_Debug_Chr                       0x000100c3
#define In_Debug_Chr                        0x000100c4
#define Debug_Convert_Hex_Binary            0x000100c5
#define Debug_Convert_Hex_Decimal           0x000100c6
#define Debug_Test_Valid_Handle             0x000100c7
#define Validate_Client_Ptr                 0x000100c8
#define Test_Reenter                        0x000100c9
#define Queue_Debug_String                  0x000100ca
#define Log_Proc_Call                       0x000100cb
#define Debug_Test_Cur_VM                   0x000100cc
#define Get_PM_Int_Type                     0x000100cd
#define Set_PM_Int_Type                     0x000100ce
#define Get_Last_Updated_System_Time        0x000100cf
#define Get_Last_Updated_VM_Exec_Time       0x000100d0
#define Test_DBCS_Lead_Byte                 0x000100d1
#define _AddFreePhysPage                    0x000100d2
#define _PageResetHandlePAddr               0x000100d3
#define _SetLastV86Page                     0x000100d4
#define _GetLastV86Page                     0x000100d5
#define _MapFreePhysReg                     0x000100d6
#define _UnmapFreePhysReg                   0x000100d7
#define _XchgFreePhysReg                    0x000100d8
#define _SetFreePhysRegCalBk                0x000100d9
#define Get_Next_Arena                      0x000100da
#define Get_Name_Of_Ugly_TSR                0x000100db
#define Get_Debug_Options                   0x000100dc
#define Set_Physical_HMA_Alias              0x000100dd
#define _GetGlblRng0V86IntBase              0x000100de
#define _Add_Global_V86_Data_Area           0x000100df
#define GetSetDetailedVMError               0x000100e0
#define Is_Debug_Chr                        0x000100e1
#define Clear_Mono_Screen                   0x000100e2
#define Out_Mono_Chr                        0x000100e3
#define Out_Mono_String                     0x000100e4
#define Set_Mono_Cur_Pos                    0x000100e5
#define Get_Mono_Cur_Pos                    0x000100e6
#define Get_Mono_Chr                        0x000100e7
#define Locate_Byte_In_ROM                  0x000100e8
#define Hook_Invalid_Page_Fault             0x000100e9
#define Unhook_Invalid_Page_Fault           0x000100ea
#define Set_Delete_On_Exit_File             0x000100eb
#define Close_VM                            0x000100ec
#define Enable_Touch_1st_Meg		           0x000100ed
#define Disable_Touch_1st_Meg		           0x000100ee
#define Install_Exception_Handler           0x000100ef
#define Remove_Exception_Handler            0x000100f0
#define Get_Crit_Status_No_Block            0x000100f1

//----------------------------------------------------------------------------
//
//                    Flags for page allocator calls
//
//----------------------------------------------------------------------------

#define PageZeroInit            0x00000001
#define PageUseAlign            0x00000002
#define PageContig              0x00000004
#define PageFixed               0x00000008
#define PageDEBUGNulFault       0x00000010
#define PageZeroReInit          0x00000020
#define PageNoCopy              0x00000040
#define PageLocked              0x00000080
#define PageLockedIfDP          0x00000100
#define PageSetV86Pageable      0x00000200
#define PageClearV86Pageable    0x00000400
#define PageSetV86IntsLocked    0x00000800
#define PageClearV86IntsLocked  0x00001000
#define PageMarkPageOut         0x00002000
#define PagePDPSetBase          0x00004000
#define PagePDPClearBase        0x00008000
#define PageDiscard             0x00010000
#define PagePDPQueryDirty       0x00020000

// New for 3.10

#define PageMapFreePhysReg      0x00040000

// NOTE: HIGH 8 BITS (bits 24-31) are reserved

//----------------------------------------------------------------------------
//
//                         Page table entry bits
//
//----------------------------------------------------------------------------

#define P_SIZE        0x1000  // page size

#define P_PRES        0x01    // page present bit
#define P_PRESBit     0
#define P_WRITE       0x02    // write access bit
#define P_WRITEBit    1       
#define P_USER        0x04    // access bit for User mode
#define P_USERBit     2       
#define P_ACC         0x20    // page accessed bit
#define P_ACCBit      5       
#define P_DIRTY       0x40    // page dirty bit
#define P_DIRTYBit    6

#define P_AVAIL       (P_PRES|P_WRITE|P_USER) // avail to everyone & present

//---------------------------------------------------
//
// Page types - definition of the OS reserved
//              bits in the page table entry.
//
//---------------------------------------------------

#define PG_TYPE       0x0E00  // TYPE bits in PTE

//---------------------------------------------------
//
//   Page types for page allocator calls
//
//---------------------------------------------------

#define PG_VM         0
#define PG_SYS        1
#define PG_RESERVED1  2
#define PG_PRIVATE    3
#define PG_RESERVED2  4
#define PG_RELOCK     5      // PRIVATE to MMGR
#define PG_INSTANCE   6
#define PG_HOOKED     7
#define PG_IGNORE     0x0FFFFFFFF

//---------------------------------------------------
//
//   Types for page table entries
//
//---------------------------------------------------

#define PgT_VM        (PG_VM << 9)
#define PgT_SYS       (PG_SYS << 9)
#define PgT_RESERVED1 (PG_RESERVED1 << 9)
#define PgT_PRIVATE   (PG_PRIVATE << 9)
#define PgT_RESERVED2 (PG_RESERVED2 << 9)
#define PgT_RELOCK    (PG_RELOCK << 9)
#define PgT_INSTANCE  (PG_INSTANCE << 9)
#define PgT_HOOKED    (PG_HOOKED << 9)

//----------------------------------------------------------------------------
//
//            Definitions for the access byte in a descriptor
//
//----------------------------------------------------------------------------

#define D_PRES       0x80     // present in memory
#define D_NOTPRES    0x00     // not present in memory

#define D_DPL0       0x00     // Ring 0
#define D_DPL1       0x20     // Ring 1
#define D_DPL2       0x40     // Ring 2
#define D_DPL3       0x60     // Ring 3

#define D_SEG        0x10     // Segment descriptor
#define D_CTRL       0x00     // Control descriptor

#define D_GRAN_BYTE  0x00     // Segment length is byte granular
#define D_GRAN_PAGE  0x80     // Segment length is page granular
#define D_DEF16      0x00     // Default operation size is 16 bits
#define D_DEF32      0x40     // Default operation size is 32 bits


#define D_CODE       0x08     // code
#define D_DATA       0x00     // data

#define D_RX         0x02     // if code, readable
#define D_X          0x00     // if code, exec only
#define D_W          0x02     // if data, writable
#define D_R          0x00     // if data, read only

#define D_ACCESSED   0x01     // segment accessed bit

#define RW_Data_Type (D_PRES+D_SEG+D_DATA+D_W)
#define R_Data_Type  (D_PRES+D_SEG+D_DATA+D_R)
#define Code_Type    (D_PRES+D_SEG+D_CODE+D_RX)

#define D_PAGE32     (D_GRAN_PAGE+D_DEF32)

#define SELECTOR_MASK  0xfff8      // selector index
#define SEL_LOW_MASK   0x00f8      // mask for low byte of sel indx
#define TABLE_MASK     0x0004      // table bit
#define RPL_MASK       0x0003      // priviledge bits
#define RPL_CLR        (~0x0003)   // clear ring bits

//----------------------------------------------------------------------------
//
//             Flags for _Allocate_Global_V86_Data_Area
//
//----------------------------------------------------------------------------

#define GVDAWordAlign      0x0001
#define GVDADWordAlign     0x0002
#define GVDAParaAlign      0x0004
#define GVDAPageAlign      0x0008
#define GVDAInstance       0x0100
#define GVDAZeroInit       0x0200
#define GVDAReclaim        0x0400

// New for 3.10

#define GVDAInquire        0x0800
#define GVDAHighSysCritOK  0x1000

//----------------------------------------------------------------------------
//
//                     Flags for heap allocator calls
//
//----------------------------------------------------------------------------

#define HeapZeroInit    0x0001
#define HeapZeroReInit  0x0002
#define HeapNoCopy      0x0004

//----------------------------------------------------------------------------
//
//                     Flags for List_Create
//
//----------------------------------------------------------------------------

#define LF_Async            0x0001
#define LF_Async_Bit        0
#define LF_Use_Heap         0x0002
#define LF_Use_Heap_Bit     1
#define LF_Alloc_Error      0x0004
#define LF_Alloc_Error_Bit  2

//----------------------------------------------------------------------------
//
//                         Scheduler Boost Values
//
//----------------------------------------------------------------------------

#define Reserved_Low_Boost      0x00000001
#define Cur_Run_VM_Boost        0x00000004
#define Low_Pri_Device_Boost    0x00000010
#define High_Pri_Device_Boost   0x00001000
#define Critical_Section_Boost  0x00100000
#define Time_Critical_Boost     0x00400000
#define Reserved_High_Boost     0x40000000

//----------------------------------------------------------------------------
//
//               Flags for Call_Priority_VM_Event
//
//----------------------------------------------------------------------------

#define PEF_Wait_For_STI       0x0001
#define PEF_Wait_For_STI_Bit   0
#define PEF_Wait_Not_Crit      0x0002
#define PEF_Wait_Not_Crit_Bit  1
#define PEF_Dont_Unboost       0x0004
#define PEF_Dont_Unboost_Bit   2
#define PEF_Always_Sched       0x0008
#define PEF_Always_Sched_Bit   3
#define PEF_Time_Out           0x0010
#define PEF_Time_Out_Bit       4

//----------------------------------------------------------------------------
//
//        Flags for Begin_Critical_Section and Wait_Semaphore
//
//----------------------------------------------------------------------------

#define Block_Svc_Ints                0x0001
#define Block_Svc_Ints_Bit            0
#define Block_Svc_If_Ints_Locked      0x0002
#define Block_Svc_If_Ints_Locked_Bit  1
#define Block_Enable_Ints             0x0004
#define Block_Enable_Ints_Bit         2
#define Block_Poll                    0x0008
#define Block_Poll_Bit                3

//----------------------------------------------------------------------------
//
//                            Data structures
//
//----------------------------------------------------------------------------

typedef struct tagREGS
{
    DWORD  eflags ;
    DWORD  edi ;
    DWORD  esi ;
    DWORD  ebp ;
    DWORD  esp ;
    DWORD  ebx ;
    DWORD  edx ;
    DWORD  ecx ;
    DWORD  eax ;

} REGS, *PREGS ;

#define EFLAGS_CF   0x00000001
#define EFLAGS_PF   0x00000004
#define EFLAGS_AF   0x00000010
#define EFLAGS_ZF   0x00000040
#define EFLAGS_SF   0x00000080

typedef struct tagCRS_32
{
   DWORD  Client_EDI ;
   DWORD  Client_ESI ;
   DWORD  Client_EBP ;
   DWORD  dwReserved_1 ;          // ESP at pushall
   DWORD  Client_EBX ;
   DWORD  Client_EDX ;
   DWORD  Client_ECX ;
   DWORD  Client_EAX ;
   DWORD  Client_Error ;          // DWORD error code
   DWORD  Client_EIP ;
   WORD   Client_CS ;
   WORD   wReserved_2 ;           // (padding)
   DWORD  Client_EFlags ;
   DWORD  Client_ESP ;
   WORD   Client_SS ;
   WORD   wReserved_3 ;           // (padding)
   WORD   Client_ES ;
   WORD   WReserved_4 ;           // (padding)
   WORD   Client_DS ;
   WORD   wReserved_5 ;           // (padding)
   WORD   Client_FS ;
   WORD   wReserved_6 ;           // (padding)
   WORD   Client_GS ;
   WORD   wReserved_7 ;           // (padding)

   DWORD  Client_Alt_EIP ;
   WORD   Client_Alt_CS ;
   WORD   wReserved_8 ;           // (padding)
   DWORD  Client_Alt_EFlags ;
   DWORD  Client_Alt_ESP ;
   WORD   Client_Alt_SS ;
   WORD   wReserved_9 ;           // (padding)
   WORD   Client_Alt_ES ;
   WORD   WReserved_10 ;          // (padding)
   WORD   Client_Alt_DS ;
   WORD   wReserved_11 ;          // (padding)
   WORD   Client_Alt_FS ;
   WORD   wReserved_12 ;          // (padding)
   WORD   Client_Alt_GS ;
   WORD   wReserved_13 ;          // (padding)

} CRS_32, *PCRS_32 ;

typedef struct tagCRS_16
{
   WORD   Client_DI ;
   WORD   wReserved_1 ;           // (padding)
   WORD   Client_SI ;
   WORD   wReserved_2 ;           // (padding)
   WORD   Client_BP ;
   WORD   wReserved_3 ;           // (padding)
   DWORD  dwReserved_4 ;          // ESP at pushall
   WORD   Client_BX ;
   WORD   wReserved_5 ;           // (padding)
   WORD   Client_DX ;
   WORD   wReserved_6 ;           // (padding)
   WORD   Client_CX ;
   WORD   wReserved_7 ;           // (padding)
   WORD   Client_AX ;
   WORD   wReserved_8 ;           // (padding)
   DWORD  dwReserved_9 ;          // DWORD error code
   WORD   Client_IP ;
   WORD   wReserved_10 ;          // (padding)
   WORD   Client_CS ;
   WORD   wReserved_11 ;          // (padding)
   WORD   Client_Flags ;          // flags (loword)
   WORD   wReserved_12 ;          // (padding)
   WORD   Client_SP ;
   WORD   wReserved_13 ;          // (padding)
   WORD   Client_SS ;
   WORD   wReserved_14 ;          // (padding)
   WORD   Client_ES ;
   WORD   wReserved_15 ;          // (padding)
   WORD   Client_DS ;
   WORD   wReserved_16 ;          // (padding)
   WORD   Client_FS ;
   WORD   wReserved_17 ;          // (padding)
   WORD   Client_GS ;
   WORD   wReserved_18 ;          // (padding)

   WORD   Client_Alt_IP ;
   WORD   wReserved_19 ;          // (padding)
   WORD   Client_Alt_CS ;
   WORD   wReserved_20 ;          // (padding)
   WORD   Client_Alt_Flags ;
   WORD   wReserved_21 ;          // (padding)
   WORD   Client_Alt_SP ;
   WORD   wReserved_22 ;          // (padding)
   WORD   Client_Alt_SS ;
   WORD   wReserved_23 ;          // (padding)
   WORD   Client_Alt_ES ;
   WORD   wReserved_24 ;          // (padding)
   WORD   Client_Alt_DS ;
   WORD   wReserved_25 ;          // (padding)
   WORD   Client_Alt_FS ;
   WORD   wReserved_26 ;          // (padding)
   WORD   Client_Alt_GS ;
   WORD   wReserved_27 ;          // (padding)

} CRS_16, *PCRS_16 ;

typedef struct tagCRS_8
{
   WORD   Client_DI ;
   WORD   wReserved_1 ;           // (padding)
   WORD   Client_SI ;
   WORD   wReserved_2 ;           // (padding)
   WORD   Client_BP ;
   WORD   wReserved_3 ;           // (padding)
   DWORD  dwReserved_4 ;          // ESP at pushall
   BYTE   Client_BL ;
   BYTE   Client_BH ;
   WORD   wReserved_5 ;           // (padding)
   BYTE   Client_DL ;
   BYTE   Client_DH ;
   WORD   wReserved_6 ;           // (padding)
   BYTE   Client_CL ;
   BYTE   Client_CH ;
   WORD   wReserved_7 ;           // (padding)
   BYTE   Client_AL ;
   BYTE   Client_AH ;
   WORD   wReserved_8 ;           // (padding)
   DWORD  dwReserved_9 ;          // DWORD error code
   WORD   Client_IP ;
   WORD   wReserved_10 ;          // (padding)
   WORD   Client_CS ;
   WORD   wReserved_11 ;          // (padding)
   WORD   Client_Flags ;          // flags (loword)
   WORD   wReserved_12 ;          // (padding)
   WORD   Client_SP ;
   WORD   wReserved_13 ;          // (padding)
   WORD   Client_SS ;
   WORD   wReserved_14 ;          // (padding)
   WORD   Client_ES ;
   WORD   wReserved_15 ;          // (padding)
   WORD   Client_DS ;
   WORD   wReserved_16 ;          // (padding)
   WORD   Client_FS ;
   WORD   wReserved_17 ;          // (padding)
   WORD   Client_GS ;
   WORD   wReserved_18 ;          // (padding)

   WORD   Client_Alt_IP ;
   WORD   wReserved_19 ;          // (padding)
   WORD   Client_Alt_CS ;
   WORD   wReserved_20 ;          // (padding)
   WORD   Client_Alt_Flags ;
   WORD   wReserved_21 ;          // (padding)
   WORD   Client_Alt_SP ;
   WORD   wReserved_22 ;          // (padding)
   WORD   Client_Alt_SS ;
   WORD   wReserved_23 ;          // (padding)
   WORD   Client_Alt_ES ;
   WORD   wReserved_24 ;          // (padding)
   WORD   Client_Alt_DS ;
   WORD   wReserved_25 ;          // (padding)
   WORD   Client_Alt_FS ;
   WORD   wReserved_26 ;          // (padding)
   WORD   Client_Alt_GS ;
   WORD   wReserved_27 ;          // (padding)

} CRS_8, *PCRS_8 ;

//----------------------------------------------------------------------------
//                VM control block structure (VMM)
//----------------------------------------------------------------------------

typedef struct tagVMMCB
{
   DWORD  CB_VM_Status ;
   DWORD  CB_High_Linear ;
   DWORD  CB_Client_Pointer ;
   DWORD  CB_VMID ;

} VMMCB, *PVMMCB ;

// VM status indicates globally interesting VM states

#define VMStat_Exclusive           0x00001 // VM is exclusive mode
#define VMStat_Exclusive_Bit       0x00
#define VMStat_Background          0x00002 // VM runs in background
#define VMStat_Background_Bit      0x01
#define VMStat_Creating            0x00004 // In process of creating
#define VMStat_Creating_Bit        0x02
#define VMStat_Suspended           0x00008 // VM not scheduled
#define VMStat_Suspended_Bit       0x03
#define VMStat_Not_Executeable     0x00010 // VM partially destroyed
#define VMStat_Not_Executeable_Bit 0x04
#define VMStat_PM_Exec             0x00020 // Currently in PM app
#define VMStat_PM_Exec_Bit         0x05
#define VMStat_PM_App              0x00040 // PM app present in VM
#define VMStat_PM_App_Bit          0x06
#define VMStat_PM_Use32            0x00080 // PM app is 32-bit
#define VMStat_PM_Use32_Bit        0x07
#define VMStat_VxD_Exec            0x00100 // Call from VxD
#define VMStat_VxD_Exec_Bit        0x08
#define VMStat_High_Pri_Back       0x00200 // High pri background
#define VMStat_High_Pri_Back_Bit   0x09
#define VMStat_Blocked             0x00400 // Blocked on semaphore
#define VMStat_Blocked_Bit         0x0A
#define VMStat_Awakening           0x00800 // Woke up after blocked
#define VMStat_Awakening_Bit       0x0B
#define VMStat_PageableV86         0x01000 // part of V86 is pageable (PM app)
#define VMStat_PageableV86Bit      0x0C
#define VMStat_V86IntsLocked       0x02000 // Rest of V86 is locked
#define VMStat_V86IntsLockedBit    0x0D    //    regardless of pager type
#define VMStat_TS_Sched            0x04000 // Scheduled by time-slicer
#define VMStat_TS_Sched_Bit        0x0E
#define VMStat_Idle                0x08000 // VM has released time slice
#define VMStat_Idle_Bit            0x0F
#define VMStat_Closing             0x10000 // Close_VM called for VM
#define VMStat_Closing_Bit         0x10

#define VMStat_Use32_Mask          (VMStat_PM_Use32 | VMStat_VxD_Exec)

//----------------------------------------------------------------------------
//                Device descriptor block structure
//----------------------------------------------------------------------------

#define DDK_Version 0x30A

typedef struct tagVxD_Desc_Block
{
   DWORD  DDB_Next ;                // VMM reserved field
   WORD   DDB_SDK_Version  ;        // VMM reserved field
   WORD   DDB_Req_Device_Number ;   // Required device number
   BYTE   DDB_Dev_Major_Version ;   // Major device number
   BYTE   DDB_Dev_Minor_Version ;   // Minor device number
   WORD   DDB_Flags ;               // Flags for init calls complete
   BYTE   DDB_Name[ 8 ] ;           // Device name
   DWORD  DDB_Init_Order ;          // Initialization Order
   DWORD  DDB_Control_Proc ;        // Offset of control procedure
   DWORD  DDB_V86_API_Proc ;        // Offset of API procedure
   DWORD  DDB_PM_API_Proc ;         // Offset of API procedure
   DWORD  DDB_V86_API_CSIP ;        // CS:IP of API entry point
   DWORD  DDB_PM_API_CSIP ;         // CS:IP of API entry point
   DWORD  DDB_Reference_Data ;      // Reference data from real mode
   DWORD  DDB_Service_Table_Ptr ;   // Pointer to service table
   DWORD  DDB_Service_Table_Size ;  // Number of services

} DDB ;

//----------------------------------------------------------------------------
//               Data structures for VxD service tables
//----------------------------------------------------------------------------

typedef VOID (CDECL* SERVICEPROC)( PREGS ) ;
typedef struct tagSERVICETABLE
{
   SERVICEPROC Services[] ;

} SERVICETABLE, *PSERVICETABLE ;

//----------------------------------------------------------------------------
//             Data structures for vmmInstallMultIOHandlers
//----------------------------------------------------------------------------

typedef UINT (CDECL* VXDIOPROC)( DWORD, UINT, UINT, PCRS_32, UINT ) ;

typedef struct tagVXDIOENTRY
{
   WORD       VxD_IO_Port ;
   VXDIOPROC  VxD_IO_Proc ;

} VXDIOENTRY, *PVXDIOENTRY ;

typedef struct tagVXDIOTABLE
{
   WORD        IOT_Ports ;
   VXDIOENTRY  IOT_Entries[] ;

} VXDIOTABLE, *PVXDIOTABLE ;

//----------------------------------------------------------------------------
//                Data structure for vmmAddInstanceItem
//----------------------------------------------------------------------------

typedef struct tagINSTDATASTRUC
{
   DWORD   dwInstLinkF ;      // reserved set to NULL
   DWORD   dwInstLinkB ;      // reserved set to NULL
   DWORD   dwInstLinAddr ;    // linear address of start of block
   DWORD   dwInstSize ;       // size of block in bytes
   DWORD   dwInstType ;       // type of block

} INSTDATASTRUC, *PINSTDATASTRUC ;

// values for dwInstType

#define INDOS_Field   0x0100
#define ALWAYS_Field  0x0200

//----------------------------------------------------------------------------
//                Data structure for vmmAddInstanceItem
//----------------------------------------------------------------------------

typedef struct tagIPFDATA
{

   DWORD  IPF_LinAddr ;       // CR2 address of fault
   DWORD  IPF_MapPageNum ;    // Possible converted page # of fault
   DWORD  IPF_PTEEntry ;      // Contents of PTE that faulted
   DWORD  IPF_FaultingVM ;    // May not = Current VM (IPF_V86PgH set)
   DWORD  IPF_Flags ;         // Flags

} IPFDATA, *PIPFDATA ;

// data types for message dispatch table used by VMMWRAP.ASM

typedef BOOL (CDECL* DISPATCHPROC)( DWORD, ... ) ;
typedef struct tagDISPATCHINFO
{
   DWORD         dwMsg ;
   DISPATCHPROC  lpProc ;

} DISPATCHINFO ;

// data type for v86 interrupt callbacks

typedef BOOL (CDECL* V86INTPROC)( DWORD, UINT, PCRS_32 ) ;

// data type for v86 page hook

typedef VOID (CDECL* V86PHPROC)( DWORD, UINT ) ;

// data type for invalid page fault callbacks

typedef BOOL (CDECL* IPFPROC)( DWORD, PIPFDATA ) ;

// data type for event callbacks

typedef VOID (CDECL* EVENTPROC)( DWORD, DWORD, PCRS_32 ) ;

// data type for API procedures

typedef VOID (CDECL* APIPROC)( DWORD, PCRS_32 ) ;

// function prototypes

VOID PASCAL vmmwrapVxDControlProc( VOID ) ;
DWORD PASCAL vmmwrapThunkVDMADProc( DWORD ) ;
VXDIOPROC PASCAL vmmwrapThunkVXDIOProc( VXDIOPROC ) ;
V86INTPROC PASCAL vmmwrapThunkV86IntProc( V86INTPROC ) ;
V86PHPROC PASCAL vmmwrapThunkV86PHProc( V86PHPROC ) ;
IPFPROC PASCAL vmmwrapThunkIPFProc( IPFPROC ) ;
EVENTPROC PASCAL vmmwrapThunkEventProc( EVENTPROC ) ;
APIPROC PASCAL vmmwrapThunkAPIProc( APIPROC ) ;

UINT PASCAL vmmGetVMMVersion( VOID ) ;
UINT PASCAL vmmGetCurVMHandle( VOID ) ;
BOOL PASCAL vmmTestCurVMHandle( UINT ) ;
UINT PASCAL vmmGetSysVMHandle( VOID ) ;
BOOL PASCAL vmmTestSysVMHandle( UINT ) ;
BOOL PASCAL vmmValidateVMHandle( UINT ) ;

UINT PASCAL vmmGetLastUpdatedSystemTime( VOID ) ;

VOID PASCAL vmmBeginCriticalSection( DWORD, UINT ) ;
VOID PASCAL vmmEndCriticalSection( DWORD ) ;

VOID PASCAL vmmGetV86IntVector( DWORD, UINT, PWORD, PDWORD ) ;

UINT PASCAL vmmAllocateDeviceCBArea( UINT, UINT ) ;
BOOL PASCAL vmmAddInstanceItem( PINSTDATASTRUC, UINT ) ;
VOID PASCAL vmmBuildDescriptorDWORDs( PDWORD, PDWORD, PSTR, UINT,
                                      UINT, UINT, UINT ) ;
UINT PASCAL vmmAllocateGDTSelector( DWORD, DWORD, UINT ) ;
UINT PASCAL vmmMapPhysToLinear( DWORD, UINT, UINT ) ;
BOOL PASCAL vmmPhysIntoV86( DWORD, DWORD, DWORD, UINT, UINT ) ;
UINT PASCAL vmmPageAllocate( PDWORD, UINT, UINT, UINT, UINT, UINT, UINT,
                              PDWORD, UINT ) ;
UINT PASCAL vmmPageReAllocate( PDWORD, DWORD, UINT, UINT ) ;
BOOL PASCAL vmmPageFree( DWORD, UINT ) ;
BOOL PASCAL vmmPageLock( DWORD, UINT, UINT, UINT ) ;
BOOL PASCAL vmmPageUnLock( DWORD, UINT, UINT, UINT ) ;

DWORD PASCAL vmmHeapAllocate( UINT, UINT ) ;
DWORD PASCAL vmmHeapReAllocate( DWORD, UINT, UINT ) ;

BOOL PASCAL vmmCopyPageTable( UINT, UINT, PDWORD, UINT ) ;

UINT PASCAL vmmGetFirstV86Page( DWORD ) ;
UINT PASCAL vmmGetLastV86Page( DWORD ) ;
BOOL PASCAL vmmSetLastV86Page( UINT, DWORD ) ;

BOOL PASCAL vmmGetDeviceV86PagesArray( DWORD, PDWORD, UINT ) ;
BOOL PASCAL vmmAssignDeviceV86Pages( UINT, UINT, DWORD, UINT ) ;

DWORD PASCAL vmmListCreate( UINT, UINT ) ;
DWORD PASCAL vmmListGetNext( DWORD, DWORD ) ;

BOOL PASCAL vmmInstallMultIOHandlers( PVXDIOTABLE ) ;
VOID PASCAL vmmEnableLocalTrapping( DWORD, UINT ) ;
VOID PASCAL vmmDisableLocalTrapping( DWORD, UINT ) ;

VOID PASCAL vmmSaveClientState( DWORD, PCRS_32, PCRS_32 ) ;
VOID PASCAL vmmRestoreClientState( DWORD, PCRS_32, PCRS_32 ) ;

VOID PASCAL vmmBeginNestExec( DWORD, PCRS_32 ) ;
VOID PASCAL vmmBeginNestV86Exec( DWORD, PCRS_32 ) ;
VOID PASCAL vmmEndNestExec( DWORD, PCRS_32 ) ;

VOID PASCAL vmmExecInt( DWORD, PCRS_32, UINT ) ;

DWORD PASCAL vmmGetNulPageHandle( VOID ) ;
BOOL PASCAL vmmMapIntoV86( DWORD, DWORD, UINT, UINT, UINT, UINT ) ;
BOOL PASCAL vmmHookInvalidPageFault( IPFPROC ) ;
BOOL PASCAL vmmHookV86IntChain( UINT, V86INTPROC ) ;
BOOL PASCAL vmmHookV86Page( UINT, V86PHPROC ) ;
BOOL PASCAL vmmModifyPageBits( DWORD, UINT, UINT, DWORD, DWORD,
                                DWORD, DWORD ) ;

DWORD PASCAL vmmAllocateGlobalV86DataArea( UINT, UINT ) ;

#define GPE_NOKEY    -1
#define GPE_NOVALUE  -2

BOOL PASCAL vmmGetProfileBoolean( PSTR, PSTR, BOOL, PINT ) ;
UINT PASCAL vmmGetProfileHexInt( PSTR, PSTR, UINT, PINT ) ;
UINT PASCAL vmmGetProfileDecimalInt( PSTR, PSTR, UINT, PINT ) ;

VOID PASCAL vmmDebugOut( PSTR ) ;
VOID PASCAL vmmTraceOut( PSTR ) ;
VOID PASCAL vmmTraceOutParam( PSTR, DWORD ) ;

DWORD PASCAL VDMAD_VirtualizeChannel( DWORD, DWORD ) ;

//---------------------------------------------------------------------------
//  End of File: vmm.h
//---------------------------------------------------------------------------

