        page    60, 132

;******************************************************************************
	title	VMMWRAP - VMM wrapper functions for 'C' interface
;******************************************************************************
;
;   (C) Copyright Woodruff Software Systems, 1993
;
;   Title:    VMMWRAP - A 'C' wrapper for VMM services/control
;
;   Module:   VMMWRAP.ASM - Core code
;
;   Version:  1.00
;
;   Date:     March 8, 1993
;
;   Author:   Bryan A. Woodruff
;
;******************************************************************************
;
;   Change log:
;
;   DATE      REVISION  DESCRIPTION                                 AUTHOR
;   --------  --------	------------------------------------------  -----------
;    3/ 8/93  1.00      Wrote it.                                   BryanW
;
;******************************************************************************
;
;   Functional Description:
;
;      Provides a 'C' interface to VMM control procedures/services
;
;******************************************************************************

        .386p

;==============================================================================
;                     I N C L U D E S   &   E Q U A T E S
;==============================================================================

        .XLIST
        INCLUDE VMM.Inc
        INCLUDE Debug.Inc
        .LIST

;==============================================================================
;                      N O N P A G E A B L E   D A T A
;==============================================================================

VxD_LOCKED_DATA_SEG

EXTRN alpVxDDispatchProcs:DWORD

;
; The "thunk table" is used to create headers for 'C' routines
; that are used with DMA trap handling, IO trap handling,
; API entry points, fault handlers, IRQ virtualization procedures, etc.
;

hThunkTable     dd      0
pThunkTable     dd      0
pCurrentThunk   dd      0

VxD_LOCKED_DATA_ENDS

VxD_LOCKED_CODE_SEG

;==============================================================================
;                      N O N P A G E A B L E   C O D E
;==============================================================================

;------------------------------------------------------------------------------
;
;   vmmwrapVxDControlProc
;
;   DESCRIPTION:
;       Dispatches VMM control messages to the appropriate handlers.
;
;   ENTRY:
;       EAX = Message
;       EBX = VM associated with message
;
;   EXIT:
;       Carry clear if no error (or if not handled by the VxD)
;       or set to indicate failure if the message can be failed.
;
;   USES:
;       All registers.
;
;------------------------------------------------------------------------------

BeginProc vmmwrapVxDControlProc, PUBLIC

        push    esi
        lea     esi, alpVxDDispatchProcs

VCP_Loop:
        cmp     dword ptr [esi], -1
        je      VCP_Not_Here
        cmp     eax, dword ptr [esi]
        je      SHORT VCP_Dispatch_It
        add     esi, 8
        jmp     SHORT VCP_Loop

VCP_Dispatch_It:
        cmp     eax, Sys_Critical_Init          ; Q: Sys_Critical_Init?
        jnz     SHORT VCP_Check_CmdTail         ;    N: check CmdTail frames
        mov     eax, [esi+4]                    ;    Y: special frame
        pop     esi
        push    ebp                             ; pointer to CRS
        push    esi                             ; command tail
        push    edx                             ; reference data
        push    ebx                             ; Sys VM handle
        call    eax
        add     esp, 16
        jmp     SHORT VCP_Dispatch_Complete

VCP_Check_CmdTail:
        cmp     eax, Device_Init                ; Q: Device_Init?
        jz      SHORT VCP_Dispatch_CmdTail      ;    Y: Ok
        cmp     eax, Init_Complete              ; Q: Init_Complete?
        jnz     SHORT VCP_Check_BPA             ;    N: Try again.

VCP_Dispatch_CmdTail:
        mov     eax, [esi+4]
        pop     esi
        push    ebp                             ; pointer to CRS
        push    esi                             ; command tail
        push    ebx                             ; Sys VM handle
        call    eax
        add     esp, 12
        jmp     SHORT VCP_Dispatch_Complete

VCP_Check_BPA:
        cmp     eax, Begin_PM_App               ; Q: Begin_PM_App?
        jnz     SHORT VCP_Check_SDF             ;    N: Try again.
        mov     eax, [esi+4]
        pop     esi
        push    ebp                             ; pointer to CRS
        push    edi                             ; ptr to Application CB
        push    edx                             ; flags
        push    ebx                             ; current VM handle
        call    eax
        add     esp, 16
        jmp     SHORT VCP_Dispatch_Complete

VCP_Check_SDF:
        cmp     eax, Set_Device_Focus           ; Q: Set_Device_Focus?
        jnz     SHORT VCP_Dispatch_Other        ;    N: Dispatch normal
        mov     eax, [esi+4]
        pop     esi
        push    edi                             ; associated VM
        push    esi                             ; flags
        push    edx                             ; device id to receive focus
        push    ebx                             ; VM handle
        call    eax
        add     esp, 16
        jmp     SHORT VCP_Dispatch_Complete

VCP_Dispatch_CheckParams:
        cmp     eax, Debug_Query
        je      SHORT VCP_Dispatch_CRS_Only
        cmp     eax, Sys_Critical_Exit
        jne     SHORT VCP_Dispatch_Other

VCP_Dispatch_SCE:
        mov     eax, [esi+4]
        pop     esi
        push    ebx
        call    eax
        add     esp, 4
        jmp     SHORT VCP_Exit_No_Fail

VCP_Dispatch_CRS_Only:
        mov     eax, [esi+4]
        pop     esi
        push    ebp                             ; pointer to CRS
        call    eax
        add     esp, 4
        jmp     SHORT VCP_Dispatch_Complete

VCP_Dispatch_Other:
        mov     eax, [esi+4]
        pop     esi     
        push    ebp                             ; pointer to CRS
        push    ebx                             ; current VM handle
        call    eax
        add     esp, 8

VCP_Dispatch_Complete:
        or      eax, eax
        jnz     SHORT VCP_Exit_No_Fail
        stc
        ret

VCP_Not_Here:
        pop     esi

VCP_Exit_No_Fail:
        clc
        ret

EndProc vmmwrapVxDControlProc

;------------------------------------------------------------------------------
;
;   Allocate_Procedure_Thunk
;
;   DESCRIPTION:
;       Allocates a procedure thunk from the thunk table.
;
;   ENTRY:
;       NOTHING.
;
;   EXIT:
;       if Carry clear
;          success, EAX = Thunk address
;       else
;          failure
;
;   USES:
;       EAX, Flags
;
;------------------------------------------------------------------------------

Max_ThunkTablePages     equ     2
Size_ThunkTableEntry    equ     20h

BeginProc Allocate_Procedure_Thunk
        
        push    edx
        cmp     hThunkTable, 0
        jne     SHORT APT_TablePresent

        VMMCall _PageAllocate, <Max_ThunkTablePages, PG_SYS, 0, 0, 0, 0, 0,\
                                PageFixed+PageLocked+PageZeroInit>
        or      eax, eax
        jz      SHORT APT_Failure
        mov     hThunkTable, eax
        mov     pThunkTable, edx
        mov     pCurrentThunk, edx

APT_TablePresent:
        mov     eax, pCurrentThunk
        sub     eax, pThunkTable
        mov     edx, Max_ThunkTablePages
        shl     edx, 12
        add     eax, Size_ThunkTableEntry
        cmp     eax, edx
        ja      SHORT APT_Failure
        add     eax, pThunkTable
        mov     pCurrentThunk, eax
        sub     eax, Size_ThunkTableEntry

APT_Success:
        clc
        jmp     SHORT APT_Exit

APT_Failure:
        Debug_Out "Unable to allocate procedure thunk!"

        stc

APT_Exit:
        pop     edx
        ret

EndProc Allocate_Procedure_Thunk

;------------------------------------------------------------------------------
;
;   DWORD PASCAL vmmwrapThunkVDMADProc( DWORD pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for Virtualized DMA callback procedures.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            BOOL CDECL VDMADProc( DWORD hVM, UINT hDMAChannel )
;
;   RETURN VALUE:
;      DWORD
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc vmmwrapThunkVDMADProc, PUBLIC
                
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VDProc_Failure
        jmp     SHORT VDProc_CreateThunk

;==================
; Begin thunk code

VDMADThunk label   byte

        push    eax                             ; save for later
        push    eax
        push    ebx

        call    $
VDMADThunkCallAddr equ $-VDMADThunk

        add     esp, 8                          ; fixup for CDECL
        or      eax, eax
        pop     eax                             ; restore eax
        jz      SHORT VDMADThunk_Failure
        clc
        ret

VDMADThunk_Failure:
        stc
        ret

VDMADThunkSize    equ     $-VDMADThunk

;
;End thunk code
;==================

VDProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, VDMADThunk
        mov     edi, eax
        mov     ecx, VDMADThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, VDMADThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VDProc_Exit


VDProc_Failure:
        xor     eax, eax

VDProc_Exit:
        pop     ebp
        ret     4

EndProc vmmwrapThunkVDMADProc

;------------------------------------------------------------------------------
;
;   VXDIOPROC PASCAL vmmwrapThunkVXDIOProc( VXDIOPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for VxD I/O callbacks.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            UINT CDECL VXDIOProc( DWORD hVM, UINT uIOType,
;                                  UINT uPort, PCRS_32 pCRS,
;                                  UINT uData ) 
;
;   RETURN VALUE:
;      VXDIOPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc vmmwrapThunkVXDIOProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VIProc_Failure
        jmp     SHORT VIProc_CreateThunk

;==================
; Begin thunk code

VXDIOThunk label   byte

        push    eax                             ; uData
        push    ebp                             ; pCRS
        push    edx                             ; uPort
        push    ecx                             ; uIOType
        push    ebx                             ; hVM

        call    $
VXDIOThunkCallAddr equ $-VXDIOThunk
        add     esp, 20                         ; fixup for CDECL

        clc
        ret

VXDIOThunkSize    equ     $-VXDIOThunk

;
;End thunk code
;==================

VIProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, VXDIOThunk
        mov     edi, eax
        mov     ecx, VXDIOThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, VXDIOThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VIProc_Exit


VIProc_Failure:
        xor     eax, eax

VIProc_Exit:
        pop     ebp
        ret     4

EndProc vmmwrapThunkVXDIOProc

;------------------------------------------------------------------------------
;
;   V86INTPROC PASCAL vmmwrapThunkV86IntProc( V86INTPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for V86 interrupt handler callbacks.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            BOOL CDECL V86IntProc( DWORD hVM, UINT uInt, PCRS_32 pCRS )
;
;   RETURN VALUE:
;      V86INTPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc vmmwrapThunkV86IntProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VVIProc_Failure
        jmp     SHORT VVIProc_CreateThunk

;==================
; Begin thunk code

V86IntThunk label   byte

        push    eax
        push    ebp                             ; pCRS
        push    eax                             ; uInt
        push    ebx                             ; hVM

        call    $
V86IntThunkCallAddr equ $-V86IntThunk
        add     esp, 12                         ; fixup for CDECL
        or      eax, eax
        pop     eax                             ; restore eax
        jz      SHORT V86IntThunk_Failure
        clc
        ret

V86IntThunk_Failure:
        stc
        ret

V86IntThunkSize    equ     $-V86IntThunk

;
;End thunk code
;==================

VVIProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, V86IntThunk
        mov     edi, eax
        mov     ecx, V86IntThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, V86IntThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VVIProc_Exit


VVIProc_Failure:
        xor     eax, eax

VVIProc_Exit:
        pop     ebp
        ret     4

EndProc vmmwrapThunkV86IntProc

;------------------------------------------------------------------------------
;
;   V86PHPROC PASCAL vmmwrapThunkV86PHProc( V86PHPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for VxD page fault callbacks.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            VOID CDECL V86PHProc( DWORD hVM, UINT uPage )
;
;   RETURN VALUE:
;      V86PHPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc vmmwrapThunkV86PHProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VHProc_Failure
        jmp     SHORT VHProc_CreateThunk

;==================
; Begin thunk code

V86PHThunk label   byte

        pushad
        push    eax                             ; uPage
        push    ebx                             ; hVM

        call    $
V86PHThunkCallAddr equ $-V86PHThunk
        add     esp, 8                          ; fixup for CDECL
        popad

        clc
        ret

V86PHThunkSize    equ     $-V86PHThunk

;
;End thunk code
;==================

VHProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, V86PHThunk
        mov     edi, eax
        mov     ecx, V86PHThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, V86PHThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VHProc_Exit


VHProc_Failure:
        xor     eax, eax

VHProc_Exit:
        pop     ebp
        ret     4

EndProc vmmwrapThunkV86PHProc

;------------------------------------------------------------------------------
;
;   IPFPROC PASCAL vmmwrapThunkIPFProc( IPFPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for invalid page fault callbacks.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            BOOL CDECL IPFProc( DWORD hVM, PIPFDATA pIPFData )
;
;   RETURN VALUE:
;      IPFPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc vmmwrapThunkIPFProc, PUBLIC
        
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT IPFProc_Failure
        jmp     SHORT IPFProc_CreateThunk

;==================
; Begin thunk code

IPFThunk label   byte

        push    eax
        push    edi                             ; pIPFData
        push    ebx                             ; hVM

        call    $
IPFThunkCallAddr equ $-IPFThunk
        add     esp, 8                          ; fixup for CDECL
        or      eax, eax
        pop     eax
        jnz     SHORT IPFThunk_Exit

        stc                                     ; signal not handled

IPFThunk_Exit:
        ret

IPFThunkSize    equ     $-IPFThunk

;
;End thunk code
;==================

IPFProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, IPFThunk
        mov     edi, eax
        mov     ecx, IPFThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, IPFThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT IPFProc_Exit


IPFProc_Failure:
        xor     eax, eax

IPFProc_Exit:
        pop     ebp
        ret     4

EndProc vmmwrapThunkIPFProc

;------------------------------------------------------------------------------
;
;   EVENTPROC PASCAL vmmwrapThunkEventProc( EVENTPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for VxD generic event callbacks.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            VOID CDECL EventProc( DWORD hVM, DWORD dwRefData, PCRS_32 pCRS )
;
;   RETURN VALUE:
;      EVENTPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc vmmwrapThunkEventProc, PUBLIC
        
        pCRS    equ     [ebp]
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VEProc_Failure
        jmp     SHORT VEProc_CreateThunk

;==================
; Begin thunk code

EventThunk label   byte

        push    pCRS
        push    edx                             ; reference data
        push    ebx                             ; hVM

        call    $
EventThunkCallAddr equ $-EventThunk
        add     esp, 12                         ; fixup for CDECL
        ret

EventThunkSize    equ     $-EventThunk

;
;End thunk code
;==================

VEProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, EventThunk
        mov     edi, eax
        mov     ecx, EventThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, EventThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VEProc_Exit


VEProc_Failure:
        xor     eax, eax

VEProc_Exit:
        pop     ebp
        ret     4

EndProc vmmwrapThunkEventProc

;------------------------------------------------------------------------------
;
;   APIPROC PASCAL vmmwrapThunkAPIProc( APIPROC pProc )
;
;   DESCRIPTION:
;      Creates a procedure thunk for VxD API procedures.
;
;   PARAMETERS:
;      DWORD pProc
;         pointer to callback procedure, must have the form:
;            VOID CDECL APIProc( DWORD hVM, PCRS_32 pCRS )
;
;   RETURN VALUE:
;      APIPROC
;         pointer to thunk or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc vmmwrapThunkAPIProc, PUBLIC
        
        pCRS    equ     [ebp]
        pProc   equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        call    Allocate_Procedure_Thunk
        jc      SHORT VAProc_Failure
        jmp     SHORT VAProc_CreateThunk

;==================
; Begin thunk code

APIThunk label   byte

        push    pCRS
        push    ebx                             ; hVM

        call    $
APIThunkCallAddr equ $-APIThunk
        add     esp, 8                          ; fixup for CDECL
        ret

APIThunkSize    equ     $-APIThunk

;
;End thunk code
;==================

VAProc_CreateThunk:
        push    ecx
        push    edi
        push    esi

        ;
        ; Copy the thunk...
        ;

        lea     esi, APIThunk
        mov     edi, eax
        mov     ecx, APIThunkSize
        cld
        shr     ecx, 1
        rep     movsw
        adc     cl, cl
        rep     movsb

        ;
        ; Fix it up...
        ;

        push    eax
        add     eax, APIThunkCallAddr
        mov     esi, eax
        sub     esi, 4
        sub     eax, pProc
        neg     eax
        mov     dword ptr [esi], eax
        pop     eax

        pop     esi
        pop     edi
        pop     ecx
        jmp     SHORT VAProc_Exit


VAProc_Failure:
        xor     eax, eax

VAProc_Exit:
        pop     ebp
        ret     4

EndProc vmmwrapThunkAPIProc

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmGetVMMVersion( VOID )
;
;   DESCRIPTION:
;      Returns the VMM version.
;
;   PARAMETERS:
;      Nothing.
;
;   RETURN VALUE:
;      UINT
;         VMM version
;
;------------------------------------------------------------------------------

BeginProc vmmGetVMMVersion, PUBLIC

        VMMJmp  Get_VMM_Version

EndProc vmmGetVMMVersion

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmGetCurVMHandle( VOID )
;
;   DESCRIPTION:
;      Returns the current VM handle.
;
;   PARAMETERS:
;      Nothing.
;
;   RETURN VALUE:
;      UINT
;         Current VM handle
;
;------------------------------------------------------------------------------

BeginProc vmmGetCurVMHandle, PUBLIC

        push    ebx
        VMMCall Get_Cur_VM_Handle
        mov     eax, ebx
        pop     ebx
        ret

EndProc vmmGetCurVMHandle

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmTestCurVMHandle( UINT hVM )
;
;   DESCRIPTION:
;      Tests the provided handle against the current VM handle.
;
;   PARAMETERS:
;      UINT hVM
;         handle to VM
;
;   RETURN VALUE:
;      BOOL
;         TRUE if provided handle is current VM, otherwise FALSE
;
;------------------------------------------------------------------------------

BeginProc vmmTestCurVMHandle, PUBLIC

        hVM     equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        xor     eax, eax
        push    ebx
        mov     ebx, hVM
        VMMCall Get_Cur_VM_Handle
        pop     ebx
        jne     SHORT TCVH_Exit
        mov     eax, 1

TCVH_Exit:
        pop     ebp
        ret     4

EndProc vmmTestCurVMHandle

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmGetSysVMHandle( VOID )
;
;   DESCRIPTION:
;      Returns the system VM handle.
;
;   PARAMETERS:
;      Nothing.
;
;   RETURN VALUE:
;      UINT
;         System VM handle
;
;------------------------------------------------------------------------------

BeginProc vmmGetSysVMHandle, PUBLIC

        push    ebx
        VMMCall Get_Sys_VM_Handle
        mov     eax, ebx
        pop     ebx
        ret

EndProc vmmGetSysVMHandle

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmTestSysVMHandle( UINT hVM )
;
;   DESCRIPTION:
;      Tests the provided handle against the system VM handle.
;
;   PARAMETERS:
;      UINT hVM
;         handle to VM
;
;   RETURN VALUE:
;      BOOL
;         TRUE if provided handle is system VM, otherwise FALSE
;
;------------------------------------------------------------------------------

BeginProc vmmTestSysVMHandle, PUBLIC

        hVM     equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        xor     eax, eax
        push    ebx
        mov     ebx, hVM
        VMMCall Get_Sys_VM_Handle
        pop     ebx
        jne     SHORT TSVH_Exit
        mov     eax, 1

TSVH_Exit:
        pop     ebp
        ret     4

EndProc vmmTestSysVMHandle

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmValidateVMHandle( UINT hVM )
;
;   DESCRIPTION:
;      Validates the VM handle.
;
;   PARAMETERS:
;      UINT hVM
;         handle to VM
;
;   RETURN VALUE:
;      BOOL
;         TRUE if provided handle is valid, otherwise FALSE
;
;------------------------------------------------------------------------------

BeginProc vmmValidateVMHandle, PUBLIC

        hVM     equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        xor     eax, eax
        push    ebx
        mov     ebx, hVM
        VMMCall Validate_VM_Handle
        pop     ebx
        jc      SHORT VVH_Exit
        mov     eax, 1

VVH_Exit:
        pop     ebp
        ret     4

EndProc vmmValidateVMHandle

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmGetLastUpdatedSystemTime( VOID )
;
;   DESCRIPTION:
;      Returns the last updated system time.
;
;   PARAMETERS:
;      Nothing.
;
;   RETURN VALUE:
;      UINT
;         last updated system time
;
;------------------------------------------------------------------------------

BeginProc vmmGetLastUpdatedSystemTime, PUBLIC

        push    ebp
        mov     ebp, esp

        VMMCall Get_Last_Updated_System_Time

        pop     ebp
        ret

EndProc vmmGetLastUpdatedSystemTime

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmBeginCriticalSection( DWORD hVM, UINT uFlags )
;
;   DESCRIPTION:
;      Claims the critical section for the specified VM.  If another
;      VM owns the critical section, the current VM will be blocked.
;
;   PARAMETERS:
;      DWORD hVM
;         handle of VM
;
;      UINT uFlags
;         specfies action to take when interrupts occur while
;         the virtual machine is blocked waiting for the critical
;         section:
;           
;            Block_Enable_Ints
;            Block_Svc_If_Ints_Locked
;            Block_Svc_Ints
;
;            NOTE: Block_Poll is reserved and must not be used with
;                  this service
;
;   RETURN VALUE:
;      NOTHING
;
;------------------------------------------------------------------------------

BeginProc vmmBeginCriticalSection, PUBLIC

        uFlags  equ     [ebp + 8]
        hVM     equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        push    ebx
        push    ecx

        mov     ecx, uFlags
        mov     ebx, hVM
        VMMCall Begin_Critical_Section

        pop     ecx
        pop     ebx

        pop     ebp
        ret     8

EndProc vmmBeginCriticalSection

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmEndCriticalSection( DWORD hVM )
;
;   DESCRIPTION:
;      Releases the critical section if the specified virtual
;      machine owns the critical section and the claim count
;      decremented to zero.
;
;   PARAMETERS:
;      DWORD hVM
;         handle of VM
;
;   RETURN VALUE:
;      NOTHING
;
;------------------------------------------------------------------------------

BeginProc vmmEndCriticalSection, PUBLIC

        hVM     equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        push    ebx

        mov     ebx, hVM
        VMMCall End_Critical_Section

        pop     ebx

        pop     ebp
        ret     4

EndProc vmmEndCriticalSection

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmGetV86IntVector( DWORD hVM, UINT uInt,
;                                   PWORD pSeg, PDWORD pOff )
;
;   DESCRIPTION:
;      Returns the address of the interrupt routine for the
;      specified real-mode interrupt.
;
;   PARAMETERS:
;      DWORD hVM
;         handle of VM
;
;      UINT uInt
;         interrupt vector
;
;      PWORD pSeg
;         pointer to word to hold segment
;
;      PDWORD pOff
;         pointer to dword to hold offset
;
;   RETURN VALUE:
;      NOTHING
;
;------------------------------------------------------------------------------

BeginProc vmmGetV86IntVector, PUBLIC

        pOff    equ     [ebp + 8]
        pSeg    equ     [ebp + 12]
        uInt    equ     [ebp + 16]
        hVM     equ     [ebp + 20]

        push    ebp
        mov     ebp, esp

        pushad
        mov     ebx, hVM
        mov     eax, uInt
        VMMCall Get_V86_Int_Vector
        mov     esi, pSeg
        mov     [esi], cx
        mov     esi, pOff
        mov     [esi], edx
        popad

        pop     ebp
        ret     16

EndProc vmmGetV86IntVector

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmAllocateDeviceCBArea( UINT uSize, UINT uFlags )
;
;   DESCRIPTION:
;      Allocates a device control block area and returns the offset.
;
;   PARAMETERS:
;      UINT uSize
;         size of control block
;
;      UINT uFlags
;         must be 0
;
;   RETURN VALUE:
;      UINT
;         offset of control block from VM handle
;
;------------------------------------------------------------------------------

BeginProc vmmAllocateDeviceCBArea, PUBLIC

        uFlags  equ     [ebp + 8]
        uSize   equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        VMMCall _Allocate_Device_CB_Area, <uSize, uFlags>

        pop     ebp
        ret     8

EndProc vmmAllocateDeviceCBArea

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmAddInstanceItem( PINSTDATASTRUC pInstData, UINT uFlags )
;
;   DESCRIPTION:
;      Indentifies a region of instance data in the V86 address space.
;
;   PARAMETERS:
;      PINSTDATASTRUC pInstData
;         points to a InstDataStruc structure containing information
;         about the block of memory to instance
;
;      UINT uFlags
;         must be 0
;
;   RETURN VALUE:
;      UINT
;         new selector
;
;------------------------------------------------------------------------------

BeginProc vmmAddInstanceItem, PUBLIC

        uFlags          equ     [ebp + 8]
        pInstData       equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        VMMCall _AddInstanceItem, <pInstData, uFlags>

        pop     ebp
        ret     8

EndProc vmmAddInstanceItem

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmBuildDescriptorDWORDs( PDWORD pdwHi, PDWORD pdwLo,
;                                         PSTR pBase, UINT uLen,
;                                         UINT uType, UINT uSize,
;                                         UINT uFlags )          
;
;   DESCRIPTION:
;      Builds the descriptor parameter used in calls to
;      vmmAllocateGDTSelector and vmmAllocateLDTSelector.
;
;   PARAMETERS:
;      PDWORD pdwHi
;         "High" DWORD of selector parameter
;
;      PDWORD pdwLo
;         "Low" DWORD of selector parameter
;
;      PSTR pBase
;         32-bit base address for descriptor
;
;      UINT uLen
;         20-bit limit
;
;      UINT uType
;         Specifies the present status, DPL and type fields
;         for the descriptor. (See VDAG for more info.)
;
;      UINT uSize
;         Granularity and big/default fields for descriptor.
;
;      UINT uFlags
;         Operation flags.  (See VDAG for more info.)
;
;   RETURN VALUE:
;      *pdwHi
;         contains high-doubleword of descriptor
; 
;      *pdwLo
;         contains low-doubleword of descriptor
;
;------------------------------------------------------------------------------

BeginProc vmmBuildDescriptorDWORDs, PUBLIC

        uFlags  equ     [ebp + 8]
        uSize   equ     [ebp + 12]
        uType   equ     [ebp + 16]
        uLen    equ     [ebp + 20]
        pBase   equ     [ebp + 24]
        pdwLo   equ     [ebp + 28]
        pdwHi   equ     [ebp + 32]

        push    ebp
        mov     ebp, esp

        VMMCall _BuildDescriptorDWORDs, <pBase, uLen, uType, uSize, uFlags>
        push    esi
        mov     esi, pdwLo
        mov     [esi], eax
        mov     esi, pdwHi
        mov     [esi], edx
        pop     esi

        pop     ebp
        ret     28

EndProc vmmBuildDescriptorDWORDs

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmAllocateGDTSelector( DWORD dwHi, DWORD dwLo, UINT uFlags )
;
;   DESCRIPTION:
;      Creates a new selector and adds it to the GDT.
;
;   PARAMETERS:
;      DWORD dwHi
;         Specifies the high doubleword of the descriptor.
;
;      DWORD dwLo
;         Specifies the low doubleword of the descriptor.
;
;      UINT uFlags
;         must be 0
;
;   RETURN VALUE:
;      UINT
;         new selector
;
;------------------------------------------------------------------------------

BeginProc vmmAllocateGDTSelector, PUBLIC

        uFlags  equ     [ebp + 8]
        dwLo    equ     [ebp + 12]
        dwHi    equ     [ebp + 16]

        push    ebp
        mov     ebp, esp

        VMMCall _Allocate_GDT_Selector, <dwHi, dwLo, uFlags>

        pop     ebp
        ret     12

EndProc vmmAllocateGDTSelector

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmMapPhysToLinear( DWORD dwPhysAddr, UINT uBytes,
;                                   UINT uFlags )
;
;   DESCRIPTION:
;      Returns the linear address of the first byte in the
;      specified range of physical addresses.
;
;   PARAMETERS:
;      DWORD dwPhysAddr
;         32-bit physical address of the start of the region
;
;      UINT uBytes
;         length of bytes in the region
;
;      UINT uFlags
;         flags, must be 0
;
;   RETURN VALUE:
;      UINT
;         ring-0 linear address of first byte in range or
;         -1 if unsuccessful
;
;------------------------------------------------------------------------------

BeginProc vmmMapPhysToLinear, PUBLIC

        uFlags          equ     [ebp + 8]
        uBytes          equ     [ebp + 12]
        dwPhysAddr      equ     [ebp + 16]

        push    ebp
        mov     ebp, esp

        VMMCall _MapPhysToLinear, <dwPhysAddr, uBytes, uFlags>

        pop     ebp
        ret     12

EndProc vmmMapPhysToLinear

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmPhysIntoV86( DWORD dwPhysPage, DWORD hVM,
;                               DWORD dwLinPage, UINT uPages,
;                               UINT uFlags )
;
;   DESCRIPTION:
;      Maps the specified physical pages in the V86 address space.
;
;   PARAMETERS:
;      DWORD dwPhysPage
;         specifies the physical page number of the start of
;         the region to map
;
;      DWORD hVM
;         handle of VM for which memory is to be mapped
;
;      DWORD dwLinPage
;         specifies the linear page number in the V86 space
;         (>= 10h, <= 10Fh)
;
;      UINT uPages
;         number of pages to map
;
;      UINT uFlags
;         flags, must be 0
;
;   RETURN VALUE:
;      BOOL
;         TRUE if successful, FALSE otherwise
;
;------------------------------------------------------------------------------

BeginProc vmmPhysIntoV86, PUBLIC

        uFlags          equ     [ebp + 8]
        uPages          equ     [ebp + 12]
        dwLinPage       equ     [ebp + 16]
        hVM             equ     [ebp + 20]
        dwPhysPage      equ     [ebp + 24]

        push    ebp
        mov     ebp, esp

        VMMCall _PhysIntoV86, <dwPhysPage, hVM, dwLinPage, uPages, uFlags>

        pop     ebp
        ret     20

EndProc vmmPhysIntoV86

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmPageAllocate( PDWORD phMem, UINT uPages, UINT uType,
;                                UINT hVM, UINT uAlignMask, UINT uMinPhys,
;                                UINT uMaxPhys, PDWORD pdwPhysAddr,
;                                UINT uFlags )
;
;   DESCRIPTION:
;      Allocates a block of memory of given number of pages.
;
;   PARAMETERS:
;      PDWORD phMem
;      UINT uPages
;      UINT uType
;      UINT hVM
;      UINT uAlignMask
;      UINT uMinPhys
;      UINT uMaxPhys
;      PDWORD pdwPhysAddr
;      UINT uFlags
;
;   RETURN VALUE:
;      UINT
;         linear address of block
;
;------------------------------------------------------------------------------

BeginProc vmmPageAllocate, PUBLIC

        uFlags          equ     [ebp + 8]
        pdwPhysAddr     equ     [ebp + 12]
        uMaxPhys        equ     [ebp + 16]
        uMinPhys        equ     [ebp + 20]
        uAlignMask      equ     [ebp + 24]
        hVM             equ     [ebp + 28]
        uType           equ     [ebp + 32]
        uPages          equ     [ebp + 36]
        phMem           equ     [ebp + 40]

        push    ebp
        mov     ebp, esp

        VMMCall _PageAllocate, <uPages, uType, hVM, uAlignMask, uMinPhys,\
                                uMaxPhys, pdwPhysAddr, uFlags>
        push    esi
        mov     esi, phMem
        mov     [esi], eax
        mov     eax, edx
        pop     esi

        pop     ebp
        ret     36

EndProc vmmPageAllocate

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmPageReAllocate( PDWORD phMem, DWORD hMem, UINT uPages,
;                                  UINT uFlags )
;
;   DESCRIPTION:
;      Reallocates and optionally reinitializes an existing memory block.
;
;   PARAMETERS:
;      PDWORD phMem
;         pointer to new memory handle
;
;      DWORD hMem
;         handle to current memory block
;
;      UINT uPages
;         size of the reallocated block in pages
;
;      UINT uFlags
;         operation flags
;           
;            PageLocked
;            PageLockedIfDP
;            PageNoCopy
;            PageZeroInit
;            PageZeroReInit
;
;   RETURN VALUE:
;      UINT
;         linear address of block
;
;------------------------------------------------------------------------------

BeginProc vmmPageReAllocate, PUBLIC

        uFlags          equ     [ebp + 8]
        uPages          equ     [ebp + 12]
        hMem            equ     [ebp + 16]
        phMem           equ     [ebp + 20]

        push    ebp
        mov     ebp, esp

        VMMCall _PageReAllocate, <hMem, uPages, uFlags>
        push    esi
        mov     esi, phMem
        mov     [esi], eax
        mov     eax, edx
        pop     esi

        pop     ebp
        ret     16

EndProc vmmPageReAllocate

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmPageFree( DWORD hMem, UINT uFlags )
;
;   DESCRIPTION:
;      Frees a block of memory.
;
;   PARAMETERS:
;      DWORD hMem
;         handle to memory block
;
;      UINT uFlags
;         operation flags, must be 0
;
;   RETURN VALUE:
;      BOOL
;         FALSE if unsuccesful
;
;------------------------------------------------------------------------------

BeginProc vmmPageFree, PUBLIC

        uFlags  equ     [ebp + 8]
        hMem    equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        VMMCall _PageFree, <hMem, uFlags>

        pop     ebp
        ret     8

EndProc vmmPageFree

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmPageLock( DWORD hMem, UINT uPages,
;                            UINT uOffset, UINT uFlags )
;
;   DESCRIPTION:
;      Locks one or more pages in the specified memory block.
;
;   PARAMETERS:
;      DWORD hMem
;         handle to memory block
;
;      UINT uPages
;         number of pages to lock
;       
;      UINT uOffset
;         offset (in pages) from the start of the memory block
;
;      UINT uFlags
;         operation flags
;            PageLockedIfDP - Locks pages only if virtual page swap
;                             device uses MS-DOS or BIOS functions
;                             to write to the hardware.
;
;   RETURN VALUE:
;      BOOL
;         FALSE if unsuccesful
;
;------------------------------------------------------------------------------

BeginProc vmmPageLock, PUBLIC

        uFlags  equ     [ebp + 8]
        uOffset equ     [ebp + 12]
        uPages  equ     [ebp + 16]
        hMem    equ     [ebp + 20]

        push    ebp
        mov     ebp, esp

        VMMCall _PageLock, <hMem, uPages, uOffset, uFlags>

        pop     ebp
        ret     16

EndProc vmmPageLock

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmPageUnLock( DWORD hMem, UINT uPages,
;                              UINT uOffset, UINT uFlags )
;
;   DESCRIPTION:
;      Locks one or more pages in the specified memory block.
;
;   PARAMETERS:
;      DWORD hMem
;         handle to memory block
;
;      UINT uPages
;         number of pages to lock
;       
;      UINT uOffset
;         offset (in pages) from the start of the memory block
;
;      UINT uFlags
;         operation flags
;            PageLockedIfDP -  Unlocks pages only if virtual page swap
;                              device uses MS-DOS or BIOS functions
;                              to write to the hardware.
;
;            PageMarkPageOut - Marks pages for immediate swapping, if this
;                              services sets the lock count for the pages
;                              to zero.
;
;   RETURN VALUE:
;      BOOL
;         FALSE if unsuccesful
;
;------------------------------------------------------------------------------

BeginProc vmmPageUnLock, PUBLIC

        uFlags  equ     [ebp + 8]
        uOffset equ     [ebp + 12]
        uPages  equ     [ebp + 16]
        hMem    equ     [ebp + 20]

        push    ebp
        mov     ebp, esp

        VMMCall _PageUnLock, <hMem, uPages, uOffset, uFlags>

        pop     ebp
        ret     16

EndProc vmmPageUnLock

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmCopyPageTable( UINT uLinPageNum, UINT uPages,
;                                 PDWORD pPTEBuf, UINT uFlags )
;
;   DESCRIPTION:
;      Copies one or more PTEs into the provided buffer.
;
;   PARAMETERS:
;      UINT uLinPageNum
;         starting page number
;
;      UINT uPages
;         number of PTEs to copy
;
;      PDWORD pPTEBuf
;         points to the buffer to receive the PTEs 
;
;      UINT uFlags
;         operation flags, must be 0
;
;   RETURN VALUE:
;      BOOL
;         TRUE if successful, otherwise FALSE
;
;------------------------------------------------------------------------------

BeginProc vmmCopyPageTable, PUBLIC

        uFlags          equ     [ebp + 8]
        pPTEBuf         equ     [ebp + 12]
        uPages          equ     [ebp + 16]
        uLinPageNum     equ     [ebp + 20]

        push    ebp
        mov     ebp, esp

        VMMCall _CopyPageTable, <uLinPageNum, uPages, pPTEBuf, uFlags>

        pop     ebp
        ret     16

EndProc vmmCopyPageTable

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmHeapAllocate( UINT uBytes, UINT uFlags )
;
;   DESCRIPTION:
;      Allocates a block of memory from the heap.
;
;   PARAMETERS:
;      UINT uBytes
;         size of the block
;
;      UINT uFlags
;         Specifies the allocation flags, it can be the
;         the following value:
;
;            HeapZeroInit - Fills the memory block with zeros.
;
;   RETURN VALUE:
;      DWORD
;         linear address of block, otherwise 0 to indicate failure
;
;------------------------------------------------------------------------------

BeginProc vmmHeapAllocate, PUBLIC

        uFlags          equ     [ebp + 8]
        uBytes          equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        VMMCall _HeapAllocate, <uBytes, uFlags>

        pop     ebp
        ret     8

EndProc vmmHeapAllocate

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmHeapReAllocate( DWORD hAddress, UINT uBytes,
;                                  UINT uFlags )
;
;   DESCRIPTION:
;      Reallocates or reinitializes an existing memory block
;
;   PARAMETERS:
;      DWORD hAddress
;         specifies the address of the memory block
;
;      UINT uBytes
;         size of the block
;
;      UINT uFlags
;         Specifies the allocation flags, it can be the
;         the following value:
;
;            HeapZeroInit - Fills the memory block with zeros.
;
;   RETURN VALUE:
;      DWORD
;         linear address of block, otherwise 0 to indicate failure
;
;------------------------------------------------------------------------------

BeginProc vmmHeapReAllocate, PUBLIC

        uFlags          equ     [ebp + 8]
        uBytes          equ     [ebp + 12]
        hAddress        equ     [ebp + 16]

        push    ebp
        mov     ebp, esp

        VMMCall _HeapReAllocate, <hAddress, uBytes, uFlags>

        pop     ebp
        ret     12

EndProc vmmHeapReAllocate

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmGetFirstV86Page( DWORD hVM )
;
;   DESCRIPTION:
;      Retrieves the page number of the first page in V86 memory for
;      the current virtual machine.
;
;   PARAMETERS:
;      DWORD hVM
;         handle to VM
;
;   RETURN VALUE:
;      UINT
;         first page in V86 memory
;
;------------------------------------------------------------------------------

BeginProc vmmGetFirstV86Page, PUBLIC

        hVM     equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        push    ebx
        mov     ebx, hVM
        VMMCall _GetFirstV86Page
        pop     ebx

        pop     ebp
        ret     4

EndProc vmmGetFirstV86Page

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmGetLastV86Page( DWORD hVM )
;
;   DESCRIPTION:
;      Retrieves the page number of the last page in V86 memory for
;      the current virtual machine. (Windows 3.1 only)
;
;   PARAMETERS:
;      DWORD hVM
;         handle to VM
;
;   RETURN VALUE:
;      UINT
;         last page in V86 memory
;
;------------------------------------------------------------------------------

BeginProc vmmGetLastV86Page, PUBLIC

        hVM     equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        push    ebx
        mov     ebx, hVM
        VMMCall _GetLastV86Page
        pop     ebx

        pop     ebp
        ret     4

EndProc vmmGetLastV86Page

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmSetLastV86Page( DWORD hVM, UINT uPage, UINT uFlags )
;
;   DESCRIPTION:
;      Sets the page number of the last page in V86 memory for
;      the current virtual machine. (Windows 3.1 only)
;
;   PARAMETERS:
;      DWORD hVM
;         handle to VM
;
;      UINT uPage
;         linear page number to set as the last V86 page
;
;      UINT uFlags
;         Must be 0
;
;   RETURN VALUE:
;      BOOL
;         TRUE if successful, otherwise FALSE
;
;------------------------------------------------------------------------------

BeginProc vmmSetLastV86Page, PUBLIC

        uFlags  equ     [ebp + 8]
        uPage   equ     [ebp + 12]
        hVM     equ     [ebp + 16]

        push    ebp
        mov     ebp, esp

        push    ebx
        mov     ebx, hVM
        VMMCall _SetLastV86Page, <uPage, uFlags>
        pop     ebx

        pop     ebp
        ret     12

EndProc vmmSetLastV86Page

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmAssignDeviceV86Pages( DWORD dwVMLinPage, UINT uPages,
;                                        DWORD hVM, UINT uFlags ) ;             
;
;   DESCRIPTION:
;      Retrieves a copy of the assignment array used by the
;      _Assign_Device_V86_Pages and _DeAssign_Device_V86_Pages services.
;      VxDs may use this service to determine which regions of the V86
;      address space are currently assigned and which are available.
;
;   PARAMETERS:
;      DWORD dwVMLinPage
;         specifies the linear page number of the first page of V86
;         address space to assign.  Must be 0-10Fh.
;
;      UINT uPages
;         specifies the number of pages to assign
;
;      DWORD hVM
;         handle of VM to be associated with assignment or NULL for
;         all virtual machines
;
;      UINT uFlags
;         operation flags, must be 0
;
;   RETURN VALUE:
;      BOOL
;         TRUE if successful, otherwise FALSE
;
;------------------------------------------------------------------------------

BeginProc vmmAssignDeviceV86Pages, PUBLIC

        uFlags          equ     [ebp + 8]
        hVM             equ     [ebp + 12]
        uPages          equ     [ebp + 16]
        dwVMLinPage     equ     [ebp + 20]

        push    ebp
        mov     ebp, esp

        VMMCall _Assign_Device_V86_Pages, <dwVMLinPage, uPages, hVM, uFlags>

        pop     ebp
        ret     16

EndProc vmmAssignDeviceV86Pages

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmGetDeviceV86PagesArray( DWORD hVM, PDWORD pArrayBuf,
;                                          UINT uFlags ) ;
;
;   DESCRIPTION:
;      Retrieves a copy of the assignment array used by the
;      _Assign_Device_V86_Pages and _DeAssign_Device_V86_Pages services.
;      VxDs may use this service to determine which regions of the V86
;      address space are currently assigned and which are available.
;
;   PARAMETERS:
;      DWORD hVM
;         handle of VM or NULL for global assignment array
;
;      PDWORD pArrayBuf
;         points to the 36-byte buffer that receives the assignment array
;
;      UINT uFlags
;         operation flags, must be 0
;
;   RETURN VALUE:
;      BOOL
;         TRUE if successful, otherwise FALSE
;
;------------------------------------------------------------------------------

BeginProc vmmGetDeviceV86PagesArray, PUBLIC

        uFlags          equ     [ebp + 8]
        pArrayBuf       equ     [ebp + 12]
        hVM             equ     [ebp + 16]

        push    ebp
        mov     ebp, esp

        VMMCall _Get_Device_V86_Pages_Array, <hVM, pArrayBuf, uFlags>

        pop     ebp
        ret     12

EndProc vmmGetDeviceV86PagesArray

;------------------------------------------------------------------------------
;
;   DWORD PASCAL vmmListCreate( UINT uNodeSize, UINT uFlags )
;
;   DESCRIPTION:
;      Creates a new list structure.
;
;   PARAMETERS:
;      UINT uNodeSize
;
;      UINT uFlags
;         Specifies the creation flags, it can be a combination of
;         the following values:
;
;            LF_Alloc_Error, LF_Async, LF_Use_Heap
;
;   RETURN VALUE:
;      DWORD
;         handle to the list or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc vmmListCreate, PUBLIC

        uFlags          equ     [ebp + 8]
        uNodeSize       equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        push    esi
        push    ecx
        mov     ecx, uNodeSize
        mov     eax, uFlags

        VMMCall List_Create
        pop     ecx
        mov     eax, esi
        pop     esi
        jnc     SHORT VLC_Exit
        xor     eax, eax

VLC_Exit:
        pop     ebp
        ret     8

EndProc vmmListCreate

;------------------------------------------------------------------------------
;
;   DWORD PASCAL vmmListGetNext( DWORD hList, DWORD hNode )
;
;   DESCRIPTION:
;      Returns the next node in the list.
;
;   PARAMETERS:
;      DWORD hList
;         handle to list
;
;      DWORD hNode
;         handle to current node
;
;   RETURN VALUE:
;      DWORD
;         handle to the next item in list or NULL if failure
;
;------------------------------------------------------------------------------

BeginProc vmmListGetNext, PUBLIC

        hNode   equ     [ebp + 8]
        hList   equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        push    esi
        mov     esi, hList
        mov     eax, hNode
        VMMCall List_Get_Next
        pop     esi

        pop     ebp
        ret     8

EndProc vmmListGetNext

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmInstallMultIOHandlers( PVXDIOTABLE pIOTable )
;
;   DESCRIPTION:
;      Installs I/O callback procedures for one or more I/O ports.
;
;   PARAMETERS:
;      PVXDIOTABLE pIOTable
;         pointer to I/O table
;
;   RETURN VALUE:
;      BOOL
;         TRUE if successful, FALSE otherwise
;
;------------------------------------------------------------------------------

BeginProc vmmInstallMultIOHandlers, PUBLIC

        pIOTable        equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        push    edi
        mov     edi, pIOTable
        VMMCall Install_Mult_IO_Handlers
        pop     edi
        xor     eax, eax
        jc      SHORT IMIH_Exit
        mov     eax, 1

IMIH_Exit:
        pop     ebp
        ret     4

EndProc vmmInstallMultIOHandlers

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmEnableLocalTrapping( DWORD hVM, UINT uPort )
;
;   DESCRIPTION:
;      Enables local trapping for the specified I/O port.  This
;      applies only to the specified virtual machine.
;
;   PARAMETERS:
;      DWORD hVM
;         handle of the specified VM
;
;      UINT uPort
;         the number of the I/O port for which trapping is enabled
;
;   RETURN VALUE:
;      Nothing.
;
;------------------------------------------------------------------------------

BeginProc vmmEnableLocalTrapping, PUBLIC

        uPort   equ     [ebp + 8]
        hVM     equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        push    ebx
        push    edx

        mov     ebx, hVM
        mov     edx, uPort
        VMMCall Enable_Local_Trapping

        pop     edx
        pop     ebx

        pop     ebp
        ret     8

EndProc vmmEnableLocalTrapping

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmDisableLocalTrapping( DWORD hVM, UINT uPort )
;
;   DESCRIPTION:
;      Disables local trapping for the specified I/O port.  This
;      applies only to the specified virtual machine.
;
;   PARAMETERS:
;      DWORD hVM
;         handle of the specified VM
;
;      UINT uPort
;         the number of the I/O port for which trapping is disabled
;
;   RETURN VALUE:
;      Nothing.
;
;------------------------------------------------------------------------------

BeginProc vmmDisableLocalTrapping, PUBLIC

        uPort   equ     [ebp + 8]
        hVM     equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        push    ebx
        push    edx

        mov     ebx, hVM
        mov     edx, uPort
        VMMCall Disable_Local_Trapping

        pop     edx
        pop     ebx

        pop     ebp
        ret     8

EndProc vmmDisableLocalTrapping

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmSaveClientState( DWORD hVM, PCRS_32 pCurCRS,
;                                   PCRS_32 pSaveCRS )
;
;   DESCRIPTION:
;      Saves the client register structure from the current (pCurCRS)
;      into the save structure (pSaveCRS).
;
;   PARAMETERS:
;      PCRS_32 pSaveCRS
;         pointer to save structure
;
;      PCRS_32 pCurCRS
;         pointer to current CRS
;
;   RETURN VALUE:
;      Nothing.
;
;------------------------------------------------------------------------------

BeginProc vmmSaveClientState, PUBLIC

        pSaveCRS        equ     [ebp + 8]
        pCurCRS         equ     [ebp + 12]
        hVM             equ     [ebp + 16]

        push    ebp
        mov     ebp, esp

        push    ebx
        push    ebp
        push    edi

        mov     ebx, hVM
        mov     edi, pSaveCRS
        mov     ebp, pCurCRS
        VMMCall Save_Client_State

        pop     edi
        pop     ebp
        pop     ebx

        pop     ebp
        ret     12

EndProc vmmSaveClientState

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmRestoreClientState( DWORD hVM, PCRS_32 pCurCRS,
;                                      PCRS_32 pSaveCRS )
;
;   DESCRIPTION:
;      Restores the client register structure from the save
;      structure (pSaveCRS) into current (pCurCRS).
;
;   PARAMETERS:
;      PCRS_32 pSaveCRS
;         pointer to save structure
;
;      PCRS_32 pCurCRS
;         pointer to current CRS
;
;   RETURN VALUE:
;      Nothing.
;
;------------------------------------------------------------------------------

BeginProc vmmRestoreClientState, PUBLIC

        pSaveCRS        equ     [ebp + 8]
        pCurCRS         equ     [ebp + 12]
        hVM             equ     [ebp + 16]

        push    ebp
        mov     ebp, esp

        push    ebx
        push    ebp
        push    esi

        mov     ebx, hVM
        mov     esi, pSaveCRS
        mov     ebp, pCurCRS
        VMMCall Restore_Client_State

        pop     esi
        pop     ebp
        pop     ebx

        pop     ebp
        ret     12

EndProc vmmRestoreClientState

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmBeginNestExec( DWORD hVM, PCRS_32 pCRS )
;
;   DESCRIPTION:
;      Starts a nested execution block. 
;
;   PARAMETERS:
;      DWORD hVM
;         handle to current VM
;
;      PCRS_32 pCRS
;         pointer to current CRS
;
;   RETURN VALUE:
;      Nothing.
;
;------------------------------------------------------------------------------

BeginProc vmmBeginNestExec, PUBLIC

        pCRS    equ     [ebp + 8]
        hVM     equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        push    ebp
        push    ebx

        mov     ebx, hVM
        mov     ebp, pCRS
        VMMCall Begin_Nest_Exec

        pop     ebx
        pop     ebp

        pop     ebp
        ret     8

EndProc vmmBeginNestExec

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmBeginNestV86Exec( DWORD hVM, PCRS_32 pCRS )
;
;   DESCRIPTION:
;      Starts a nested V86 execution block. 
;
;   PARAMETERS:
;      DWORD hVM
;         handle to current VM
;
;      PCRS_32 pCRS
;         pointer to current CRS
;
;   RETURN VALUE:
;      Nothing.
;
;------------------------------------------------------------------------------

BeginProc vmmBeginNestV86Exec, PUBLIC

        pCRS    equ     [ebp + 8]
        hVM     equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        push    ebp
        push    ebx

        mov     ebx, hVM
        mov     ebp, pCRS
        VMMCall Begin_Nest_V86_Exec

        pop     ebx
        pop     ebp

        pop     ebp
        ret     8

EndProc vmmBeginNestV86Exec

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmEndNestExec( DWORD hVM, PCRS_32 pCRS )
;
;   DESCRIPTION:
;      Ends a nested execution block. 
;
;   PARAMETERS:
;      DWORD hVM
;         handle to current VM
;
;      PCRS_32 pCRS
;         pointer to current CRS
;
;   RETURN VALUE:
;      Nothing.
;
;------------------------------------------------------------------------------

BeginProc vmmEndNestExec, PUBLIC

        pCRS    equ     [ebp + 8]
        hVM     equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        push    ebp
        push    ebx

        mov     ebx, hVM
        mov     ebp, pCRS
        VMMCall End_Nest_Exec

        pop     ebx
        pop     ebp

        pop     ebp
        ret     8

EndProc vmmEndNestExec

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmExecInt( DWORD hVM, PCRS_32 pCRS, UINT uInterrupt )
;
;   DESCRIPTION:
;      Ends a nested execution block. 
;
;   PARAMETERS:
;      DWORD hVM
;         handle to current VM
;
;      PCRS_32 pCRS
;         pointer to current CRS
;
;      UINT uInterrupt
;         interrupt to execute
;
;   RETURN VALUE:
;      Nothing.
;
;------------------------------------------------------------------------------

BeginProc vmmExecInt, PUBLIC

        uInterrupt      equ     [ebp + 8]
        pCRS            equ     [ebp + 12]
        hVM             equ     [ebp + 16]

        push    ebp
        mov     ebp, esp

        push    ebp
        push    ebx
        push    eax

        mov     eax, uInterrupt
        mov     ebx, hVM
        mov     ebp, pCRS
        VMMCall Exec_Int

        pop     eax
        pop     ebx
        pop     ebp

        pop     ebp
        ret     12

EndProc vmmExecInt

;------------------------------------------------------------------------------
;
;   DWORD PASCAL vmmGetNulPageHandle( VOID ) 
;
;   DESCRIPTION:
;      Returns the memory handle of the system nul page.
;
;   PARAMETERS:
;      Nothing.
;
;   RETURN VALUE:
;      DWORD
;         handle of system nul page
;
;------------------------------------------------------------------------------

BeginProc vmmGetNulPageHandle, PUBLIC

        VMMJmp  _GetNulPageHandle

EndProc vmmGetNulPageHandle

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmMapIntoV86( DWORD hMem, DWORD hVM, UINT uVMLinPgNum,
;                              UINT uPages, UINT uPageOff, UINT uFlags )
;
;   DESCRIPTION:
;      Maps one or more pages of a memory block into the V86
;      address space of the specified virtual machine.
;
;   PARAMETERS:
;      DWORD hMem
;         specifies handle identifying the memory block to map
;      
;      DWORD hVM
;         handle of VM
;
;      UINT uVMLinPgNum
;         linear page number of a V86 address.
;
;      UINT uPages
;         number of pages to map
;
;      UINT uPageOff
;         offset in pages from the beginning of the memory block
;
;      UINT uFlags
;         operation flags
;            PageDEBUGNulFault - enables page faults for system nul pages
;
;   RETURN VALUE:
;      TRUE if successful, FALSE otherwise
;
;------------------------------------------------------------------------------

BeginProc vmmMapIntoV86, PUBLIC

        uFlags          equ     [ebp + 8]
        uPageOff        equ     [ebp + 12]
        uPages          equ     [ebp + 16]
        uVMLinPgNum     equ     [ebp + 20]
        hVM             equ     [ebp + 24]
        hMem            equ     [ebp + 28]

        push    ebp
        mov     ebp, esp

        VMMCall _MapIntoV86, <hMem, hVM, uVMLinPgNum, uPages,\
                              uPageOff, uFlags>

        pop     ebp
        ret     24

EndProc vmmMapIntoV86

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmHookInvalidPageFault( IPFPROC pHookProc )
;
;   DESCRIPTION:
;      Hooks the invalid page fault handler.
;
;   PARAMETERS:
;      IPFPROC uInterrupt
;         invalid page fault callback procedure
;
;   RETURN VALUE:
;      TRUE if successful, FALSE otherwise
;
;------------------------------------------------------------------------------

BeginProc vmmHookInvalidPageFault, PUBLIC

        pHookProc       equ     [ebp + 8]

        push    ebp
        mov     ebp, esp

        push    esi
        mov     esi, pHookProc
        VMMCall Hook_Invalid_Page_Fault
        pop     esi
        jc      SHORT HIPF_ExitFailure
        mov     eax, TRUE
        jmp     SHORT HIPF_Exit

HIPF_ExitFailure:
        xor     eax, eax

HIPF_Exit:
        pop     ebp
        ret     4

EndProc vmmHookInvalidPageFault

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmHookV86Page( UINT uPage, V86PHPROC pHookProc )
;
;   DESCRIPTION:
;      Hooks a V86 page with specified page-fault handler.
;
;   PARAMETERS:
;      UINT uPage
;         specifies the number of the V86
;
;      V86PHPROC uInterrupt
;         page callback procedure
;
;   RETURN VALUE:
;      TRUE if successful, FALSE otherwise
;
;------------------------------------------------------------------------------

BeginProc vmmHookV86Page, PUBLIC

        pHookProc       equ     [ebp + 8]
        uPage           equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        push    esi
        mov     eax, uPage
        mov     esi, pHookProc
        VMMCall Hook_V86_Page
        pop     esi
        jc      SHORT HVP_ExitFailure
        mov     eax, TRUE
        jmp     SHORT HVP_Exit

HVP_ExitFailure:
        xor     eax, eax

HVP_Exit:
        pop     ebp
        ret     8

EndProc vmmHookV86Page

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmModifyPageBits( DWORD hVM, UINT uPageNum, UINT uPages,
;                                  UINT uBitAND, UINT uBitOR, UINT uPType,
;                                  DWORD dwFlags )
;
;   DESCRIPTION:
;      Modifies the page attribute bits associated with PG_HOOKED pages
;      in V86 address space of a virtual machine.
;
;   PARAMETERS:
;      DWORD hVM
;         handle to VM
;
;      UINT uPageNum
;         number of starting linear page
;
;      UINT uPages
;         number of pages to modify
;
;      UINT uBitAND
;         specifies the AND mask
;
;      UINT uBitOR
;         specifies the OR mask
;
;      UINT uPType
;         specifies the page type
;
;      DWORD dwFlags
;         flags, must be 0
;
;   RETURN VALUE:
;      BOOL
;         TRUE if successful, FALSE otherwise
;
;------------------------------------------------------------------------------

BeginProc vmmModifyPageBits, PUBLIC

        dwFlags         equ     [ebp + 8]
        uPType          equ     [ebp + 12]
        uBitOR          equ     [ebp + 16]
        uBitAND         equ     [ebp + 20]
        uPages          equ     [ebp + 24]
        uPageNum        equ     [ebp + 28]
        hVM             equ     [ebp + 32]

        push    ebp
        mov     ebp, esp

        VMMCall _ModifyPageBits, <hVM, uPageNum, uPages, uBitAND, uBitOR,\
                                  uPType, dwFlags>
        pop     ebp
        ret     28

EndProc vmmModifyPageBits

;------------------------------------------------------------------------------
;
;   DWORD PASCAL vmmAllocateGlobalV86DataArea( UINT uBytes, UINT uFlags )
;
;   DESCRIPTION:
;      Allocates a block of memory from the global V86 data area.
;
;   PARAMETERS:
;      UINT uBytes
;         specifies the number of bytes to allocate
;
;      UINT uFlags
;         Specifies the allocation flags, it can be a combination
;         of the following values:
;
;            GVDADWordAlign
;            GVDAHighSysCritOK
;            GVDAInquire
;            GVDAInstance
;            GVDAPageAlign
;            GVDAParaAlign
;            GVDAReclaim
;            GVDAWordAlign
;            GVDAZeroInit
;
;   RETURN VALUE:
;      NULL if unsuccessful, Linear Address of block otherwise
;
;------------------------------------------------------------------------------

BeginProc vmmAllocateGlobalV86DataArea, PUBLIC

        uFlags  equ     [ebp + 8]
        uBytes  equ     [ebp + 12]

        push    ebp
        mov     ebp, esp

        VMMCall _Allocate_Global_V86_Data_Area, <uBytes, uFlags>

        pop     ebp
        ret     8

EndProc vmmAllocateGlobalV86DataArea

;------------------------------------------------------------------------------
;
;   BOOL PASCAL vmmGetProfileBoolean( PSTR pSection, PSTR pKey,
;                                     BOOL fDefault, PUINT pError )
;
;   DESCRIPTION:
;      Returns the boolean value associated with pKey from pSection
;      in the SYSTEM.INI.
;
;   PARAMETERS:
;      PSTR pSection
;         pointer to null terminated string describing the [_section_]
;
;      PSTR pKey
;         pointer to null terminated string describing the _key_=value
;
;      BOOL fDefault
;         default value
;
;      PUINT pError
;         pointer to error return
;
;   RETURN VALUE:
;      BOOL
;         TRUE or FALSE according to value associated with key or default
;
;------------------------------------------------------------------------------

BeginProc vmmGetProfileBoolean, PUBLIC

        pError          equ     [ebp + 8]
        fDefault        equ     [ebp + 12]
        pKey            equ     [ebp + 16]
        pSection        equ     [ebp + 20]

        push    ebp
        mov     ebp, esp

        push    esi
        push    edi

        mov     esi, pError
        mov     dword ptr [esi], 0
        mov     esi, pSection
        mov     edi, pKey
        mov     eax, fDefault
        VMMCall Get_Profile_Boolean
        jc      SHORT GPB_NoKey
        jnz     SHORT GPB_Exit

        ;
        ; No value associated with key
        ;

        mov     esi, pError
        mov     dword ptr [esi], -2
        jmp     SHORT GPB_Exit

GPB_NoKey:

        ;
        ; Key not found
        ; 

        mov     esi, pError
        mov     dword ptr [esi], -1

GPB_Exit:
        pop     edi
        pop     esi

        pop     ebp
        ret     16

EndProc vmmGetProfileBoolean

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmGetProfileHexInt( PSTR pSection, PSTR pKey,
;                                    UINT uDefault, PUINT pError )
;
;   DESCRIPTION:
;      Returns the integer value associated with pKey from pSection
;      in the SYSTEM.INI.
;
;   PARAMETERS:
;      PSTR pSection
;         pointer to null terminated string describing the [_section_]
;
;      PSTR pKey
;         pointer to null terminated string describing the _key_=value
;
;      UINT uDefault
;         default value
;
;      PUINT pError
;         pointer to error return
;
;   RETURN VALUE:
;      UINT
;         converted hex value associated with key or default
;
;------------------------------------------------------------------------------

BeginProc vmmGetProfileHexInt, PUBLIC

        pError          equ     [ebp + 8]
        uDefault        equ     [ebp + 12]
        pKey            equ     [ebp + 16]
        pSection        equ     [ebp + 20]

        push    ebp
        mov     ebp, esp

        push    esi
        push    edi

        mov     esi, pError
        mov     dword ptr [esi], 0
        mov     esi, pSection
        mov     edi, pKey
        mov     eax, uDefault
        VMMCall Get_Profile_Hex_Int
        jc      SHORT GPHI_NoKey
        jnz     SHORT GPHI_Exit

        ;
        ; No value associated with key
        ;

        mov     esi, pError
        mov     dword ptr [esi], -2
        jmp     SHORT GPHI_Exit

GPHI_NoKey:

        ;
        ; Key not found
        ; 

        mov     esi, pError
        mov     dword ptr [esi], -1

GPHI_Exit:
        pop     edi
        pop     esi

        pop     ebp
        ret     16

EndProc vmmGetProfileHexInt

;------------------------------------------------------------------------------
;
;   UINT PASCAL vmmGetProfileDecimalInt( PSTR pSection, PSTR pKey,
;                                        UINT uDefault, PUINT pError )
;
;   DESCRIPTION:
;      Returns the integer value associated with pKey from pSection
;      in the SYSTEM.INI.
;
;   PARAMETERS:
;      PSTR pSection
;         pointer to null terminated string describing the [_section_]
;
;      PSTR pKey
;         pointer to null terminated string describing the _key_=value
;
;      UINT uDefault
;         default value
;
;      PUINT pError
;         pointer to error return
;
;   RETURN VALUE:
;      UINT
;         value associated with key or default
;
;------------------------------------------------------------------------------

BeginProc vmmGetProfileDecimalInt, PUBLIC

        pError          equ     [ebp + 8]
        uDefault        equ     [ebp + 12]
        pKey            equ     [ebp + 16]
        pSection        equ     [ebp + 20]

        push    ebp
        mov     ebp, esp

        push    esi
        push    edi

        mov     esi, pError
        mov     dword ptr [esi], 0
        mov     esi, pSection
        mov     edi, pKey
        mov     eax, uDefault
        VMMCall Get_Profile_Decimal_Int
        jc      SHORT GPDI_NoKey
        jnz     SHORT GPDI_Exit

        ;
        ; No value associated with key
        ;

        mov     esi, pError
        mov     dword ptr [esi], -2
        jmp     SHORT GPDI_Exit

GPDI_NoKey:

        ;
        ; Key not found
        ; 

        mov     esi, pError
        mov     dword ptr [esi], -1

GPDI_Exit:
        pop     edi
        pop     esi

        pop     ebp
        ret     16

EndProc vmmGetProfileDecimalInt

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmTraceOut( PSTR pStr )
;
;   DESCRIPTION:
;       Outputs a debug string.
;
;   PARAMETERS:
;       pStr
;          pointer to debug string
;
;   RETURN VALUE:
;       NOTHING.
;
;------------------------------------------------------------------------------

BeginProc vmmTraceOut, PUBLIC

        pStr    equ     [ebp + 8]

ifdef DEBUG
        push    ebp
        mov     ebp, esp
        pushfd
        pushad
        mov     esi, pStr
        VMMCall Out_Debug_String
        popad
        popfd
        pop     ebp
endif

        ret     4

EndProc vmmTraceOut

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmTraceOutParam( PSTR pStr, DWORD dwParam )
;
;   DESCRIPTION:
;       Outputs a debug string.
;
;   PARAMETERS:
;       pStr
;          pointer to debug string
;
;       dwParam
;          parameter stuffed into EAX
;
;   RETURN VALUE:
;       NOTHING.
;
;------------------------------------------------------------------------------

BeginProc vmmTraceOutParam, PUBLIC

        dwParam equ     [ebp + 8]
        pStr    equ     [ebp + 12]

ifdef DEBUG
        push    ebp
        mov     ebp, esp
        pushfd

        ;
        ; NOTE!!!
        ;
        ; Out_Debug_String uses the PushAD reference for any
        ; registers displayed in the debug string. 
        ;

        mov     eax, dwParam
        pushad
        mov     esi, pStr
        VMMCall Out_Debug_String
        popad
        popfd
        pop     ebp
endif

        ret     8

EndProc vmmTraceOutParam

;------------------------------------------------------------------------------
;
;   VOID PASCAL vmmDebugOut( PSTR pStr )
;
;   DESCRIPTION:
;       Outputs the debug string and issues INT 1.
;
;   PARAMETERS:
;       pStr
;          pointer to debug string
;        
;
;   RETURN VALUE:
;       NOTHING.
;
;------------------------------------------------------------------------------

BeginProc vmmDebugOut, PUBLIC

        pStr    equ     [ebp + 8]

ifdef DEBUG
        push    ebp
        mov     ebp, esp
        pushfd
        pushad
        mov     esi, pStr
        VMMCall Out_Debug_String
        VMMCall Test_Debug_Installed
        jz      SHORT Skip_Int1
        int     1

Skip_Int1:
        popad
        popfd
        pop     ebp
endif

        ret     4

EndProc vmmDebugOut

VxD_LOCKED_CODE_ENDS

END

;---------------------------------------------------------------------------
;  End of File: vmmwrap.asm
;---------------------------------------------------------------------------

