//---------------------------------------------------------------------------
//
//  Module: vsdinit.c
//
//  Purpose:
//     Init code and data for VSIMPLED.
//
//  Development Team:
//     Bryan A. Woodruff
//
//  History:   Date       Author      Comment
//              3/14/93   BryanW      Wrote it.
//
//---------------------------------------------------------------------------
//
//               Copyright (c) 1993 Woodruff Software Systems.
//                          All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <vmm.h>
#include "vsimpled.h"

#pragma data_seg( "_IDATA", "ICODE" )

//=============================================================================
//                             I C O D E
//=============================================================================

//------------------------------------------------------------------------
//  BOOL VSIMPLED_Sys_Critical_Init
//
//  Description:
//     On entry, interrupts are disabled.  Critical initialization
//     for this VxD should occur here.  For example, we can read
//     settings from VMM's cached copy of the SYSTEM.INI and act
//     set up our VxD as appropriate.
//
//     This procedure is called when the VxD_Control_Proc
//     dispatches the Sys_Critical_Init notification from VMM.
//
//     We can notify VMM of failure by returning FALSE,
//     or TRUE will suggest success.
//
//  Parameters:
//     DWORD hVM
//        System VM handle
//
//     DWORD dwRefData
//        reference data passed from real-mode init
//
//     PSTR pCmdTail
//        pointer to WIN.COM's command tail
//
//     PCRS_32 pCRS
//        pointer to System VM client register structure
//
//  History:   Date       Author      Comment
//              3/ 9/93   BryanW      Wrote it.
//
//------------------------------------------------------------------------

BOOL CDECL VSIMPLED_Sys_Critical_Init
(
   DWORD   hVM,
   DWORD   dwRefData,
   PSTR    pCmdTail,
   PCRS_32 pCRS
)
{
   UNUSED_PARAM( hVM ) ;
   UNUSED_PARAM( dwRefData ) ;
   UNUSED_PARAM( pCmdTail ) ;
   UNUSED_PARAM( pCRS ) ;

   vmmDebugOut( "VSIMPLED_Sys_Critical_Init\r\n" ) ;
   return ( TRUE ) ;

} // end of VSIMPLED_Sys_Critical_Init()

//------------------------------------------------------------------------
//  BOOL VSIMPLED_Device_Init
//
//  Description:
//     This is a non-system critical initialization procedure.
//     IRQ virtualization, I/O port trapping and VM control
//     block allocation can occur here.
//
//     Again, the same return value applies... TRUE for success,
//     FALSE for error notification.
//
//  Parameters:
//     DWORD hVM
//        System VM handle
//
//     PSTR pCmdTail
//        pointer to WIN.COM's command tail
//
//     PCRS_32 pCRS
//        pointer to System VM client register structure
//
//
//  History:   Date       Author      Comment
//              3/ 9/93   BryanW      Wrote it.
//
//------------------------------------------------------------------------

BOOL CDECL VSIMPLED_Device_Init
(
   DWORD   hVM,
   PSTR    pCmdTail,
   PCRS_32 pCRS
)
{

   UNUSED_PARAM( hVM ) ;
   UNUSED_PARAM( pCmdTail ) ;
   UNUSED_PARAM( pCRS ) ;

   vmmTraceOut( "VSIMPLED_Device_Init\r\n" ) ;

   return ( TRUE ) ;

} // end of VSIMPLED_Device_Init()

//---------------------------------------------------------------------------
//  End of File: vsdinit.c
//---------------------------------------------------------------------------

